# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class SpectrumOptions:
    def addam(self, af="", aa="", ab="", ac="", ad="", amin="", **kwargs):
        """Specifies the acceleration spectrum computation constants for the

        APDL Command: ADDAM
        analysis of shock resistance of shipboard structures.

        Parameters
        ----------
        af
            Direction-dependent acceleration coefficient for elastic or
            elastic-plastic analysis option (default = 0).

        aa, ab, ac, ad
            Coefficients for the DDAM acceleration spectrum equations. Default
            for these coefficients is zero.

        amin
            The minimum acceleration value in inch/sec2.  It defaults to 2316
            inch/sec2 which equals 6g, where g is acceleration due to gravity
            (g = 386 inch/sec2).

        Notes
        -----
        This command specifies acceleration coefficients to analyze shock
        resistance of shipboard equipment.  These coefficients are used to
        compute mode coefficients according to the equations given in Dynamic
        Design Analysis Method in the Mechanical APDL Theory Reference.  The
        form of these equations is based on the Naval NRL Dynamic Design
        Analysis Method.  This command, along with the VDDAM and SED commands,
        is used with the spectrum (ANTYPE,SPECTR) analysis as a special purpose
        alternative to the SV, FREQ, and SVTYP commands.  The mass and length
        units of the model must be in pounds and inches, respectively.

        DDASPEC may alternatively be used to calculate spectrum coefficients.

        This command is also valid in PREP7.
        """
        command = f"ADDAM,{af},{aa},{ab},{ac},{ad},{amin}"
        return self.run(command, **kwargs)

    def coval(
        self,
        tblno1="",
        tblno2="",
        sv1="",
        sv2="",
        sv3="",
        sv4="",
        sv5="",
        sv6="",
        sv7="",
        **kwargs,
    ):
        """Defines PSD cospectral values.

        APDL Command: COVAL

        Parameters
        ----------
        tblno1
            First input PSD table number associated with this spectrum.

        tblno2
            Second input PSD table number associated with this spectrum.

        sv1, sv2, sv3, . . . , sv7
            PSD cospectral values corresponding to the frequency points
            [PSDFRQ].

        Notes
        -----
        Defines PSD cospectral values to be associated with the previously
        defined frequency points.  Two table references are required since
        values are off-diagonal terms.  Unlike autospectra [PSDVAL], the
        cospectra can be positive or negative.  The cospectral curve segment
        where there is a sign change is interpolated linearly (the rest of the
        curve segments use log-log interpolation).  For better accuracy, choose
        as small a curve segment as possible wherever a sign change occurs.

        Repeat COVAL command using the same table numbers for additional
        points.  This command is valid for SPOPT,PSD only.

        This command is also valid in PREP7.
        """
        command = f"COVAL,{tblno1},{tblno2},{sv1},{sv2},{sv3},{sv4},{sv5},{sv6},{sv7}"
        return self.run(command, **kwargs)

    def cqc(self, signif="", label="", forcetype="", **kwargs):
        """Specifies the complete quadratic mode combination method.

        APDL Command: CQC

        Parameters
        ----------
        signif
            Combine only those modes whose significance level exceeds the
            SIGNIF threshold.  For single point, multipoint, or DDAM response
            (SPOPT,SPRS, MPRS or DDAM), the significance level of a mode is
            defined as the mode coefficient of the mode, divided by the maximum
            mode coefficient of all modes.  Any mode whose significance level
            is less than SIGNIF is considered insignificant and is not
            contributed to the mode combinations.  The higher the SIGNIF
            threshold, the fewer the number of modes combined.  SIGNIF defaults
            to 0.001.  If SIGNIF is specified as 0.0, it is taken as 0.0.
            (This mode combination method is not valid for SPOPT,PSD.)

        label
            Label identifying the combined mode solution output.

            DISP - Displacement solution (default).  Displacements, stresses, forces, etc., are
                   available.

            VELO - Velocity solution.  Velocities, "stress velocities," "force velocities," etc.,
                   are available.

            ACEL - Acceleration solution.  Accelerations, "stress accelerations," "force
                   accelerations," etc., are available.

        forcetype
            Label identifying the forces to be combined:

            STATIC - Combine the modal static forces (default).

            TOTAL - Combine the modal static plus inertial forces.

        Notes
        -----
        Damping is required for this mode combination method.  The CQC command
        is also valid for PREP7.
        """
        command = f"CQC,{signif},{label},{forcetype}"
        return self.run(command, **kwargs)

    def ddaspec(self, keyref="", shptyp="", mountloc="", deftyp="", amin="", **kwargs):
        """APDL Command: DDASPEC

        Specifies the shock spectrum computation constants for DDAM analysis.

        Parameters
        ----------
        keyref
            Key for reference catalog:

            1 - The spectrum computation constants are based on NRL-1396 (default). For more
                information, see Dynamic Design Analysis Method in the
                Mechanical APDL Theory Reference

        shptyp
            Select the ship type:

            SUBM - Submarine

            SURF - Surface ship

        mountloc
            Select the mounting location:

            HULL - Hull mounting location. These structures are mounted directly to basic hull
                   structures like frames, structural bulkheads below the water
                   line, and shell plating above the water line.

            DECK - Deck mounting location. These structures are mounted directly to decks, non-
                   structural bulkheads, or to structural bulkheads above the
                   water line.

            SHEL - Shell plating mounting location. These structures are mounted directly to shell
                   plating below the water line without intervening
                   foundations.

        deftyp
            Select the deformation type:

            ELAS - Elastic deformation (default)

            PLAS - Elastic-plastic deformation

        amin
            Minimum acceleration value in inch/sec2. It defaults to 2316
            inch/sec2 which equals 6g, where g is the acceleration due to
            gravity (g = 386 in/sec2).

        Notes
        -----
        The excitation direction is required to calculate the spectrum
        coefficients. Issue the SED command before issuing DDASPEC.

        ADDAM and VDDAM may alternatively be used to calculate spectrum
        coefficients.

        This command is also valid in PREP7.
        """
        command = f"DDASPEC,{keyref},{shptyp},{mountloc},{deftyp},{amin}"
        return self.run(command, **kwargs)

    def dsum(self, signif="", label="", td="", forcetype="", **kwargs):
        """Specifies the double sum mode combination method.

        APDL Command: DSUM

        Parameters
        ----------
        signif
            Combine only those modes whose significance level exceeds the
            SIGNIF threshold.  For single point, multipoint, or DDAM response
            (SPOPT, SPRS, MPRS, or DDAM), the significance level of a mode is
            defined as the mode coefficient of the mode, divided by the maximum
            mode coefficient of all modes.  Any mode whose significance level
            is less than SIGNIF is considered insignificant and is not
            contributed to the mode combinations.  The higher the SIGNIF
            threshold, the fewer the number of modes combined. SIGNIF defaults
            to 0.001.  If SIGNIF is specified as 0.0, it is taken as 0.0.
            (This mode combination method is not valid for SPOPT, PSD.)

        label
            Label identifying the combined mode solution output.

            DISP - Displacement solution (default).  Displacements, stresses, forces, etc., are
                   available.

            VELO - Velocity solution.  Velocities, "stress velocities," "force velocities," etc.,
                   are available.

            ACEL - Acceleration solution.  Accelerations, "stress accelerations," "force
                   accelerations," etc., are available.

        td
            Time duration for earthquake or shock spectrum.  TD defaults to 10.

        forcetype
            Label identifying the forces to be combined:

            STATIC - Combine the modal static forces (default).

            TOTAL - Combine the modal static plus inertial forces.

        Notes
        -----
        This command is also valid for PREP7.
        """
        command = f"DSUM,{signif},{label},{td},{forcetype}"
        return self.run(command, **kwargs)

    def freq(
        self,
        freq1="",
        freq2="",
        freq3="",
        freq4="",
        freq5="",
        freq6="",
        freq7="",
        freq8="",
        freq9="",
        **kwargs,
    ):
        """Defines the frequency points for the SV vs. FREQ tables.

        APDL Command: FREQ

        Parameters
        ----------
        freq1, freq2, freq3, . . . , freq9
            Frequency points for SV vs. FREQ tables.  Values must be in
            ascending order. FREQ1 should be greater than zero.  Units are
            cycles/time.

        Notes
        -----
        Repeat the FREQ command for additional frequency points (100 maximum).
        Values are added after the last nonzero frequency.  If all fields
        (FREQ1 -- FREQ9) are blank, erase SV vs. FREQ tables.

        Frequencies must be in ascending order.

        Spectral values are input with the SV command and interpreted according
        to the SVTYP command.  Applies only to the SPRS (single-point) option
        of the SPOPT command.  See the SPFREQ command for frequency input in
        MPRS (multi-point) analysis.

        Use the STAT command to list current frequency points.

        This command is also valid in PREP7.
        """
        command = f"FREQ,{freq1},{freq2},{freq3},{freq4},{freq5},{freq6},{freq7},{freq8},{freq9}"
        return self.run(command, **kwargs)

    def grp(self, signif="", label="", forcetype="", **kwargs):
        """Specifies the grouping mode combination method.

        APDL Command: GRP

        Parameters
        ----------
        signif
            Combine only those modes whose significance level exceeds the
            SIGNIF  threshold.  For single point, multipoint, or DDAM response
            (SPOPT,SPRS, MPRS or DDAM), the significance level of a mode is
            defined as the mode coefficient of the mode, divided by the maximum
            mode coefficient of all modes.  Any mode whose significance level
            is less than SIGNIF is considered insignificant and is not
            contributed to the mode combinations.  The higher the SIGNIF
            threshold, the fewer the number of modes combined.  SIGNIF defaults
            to 0.001.  If SIGNIF is specified as 0.0, it is taken as 0.0.
            (This mode combination method is not valid for SPOPT,PSD.)

        label
            Label identifying the combined mode solution output.

            DISP - Displacement solution (default).  Displacements, stresses, forces, etc., are
                   available.

            VELO - Velocity solution.  Velocities, "stress velocities," "force velocities," etc.,
                   are available.

            ACEL - Acceleration solution.  Accelerations, "stress accelerations," "force
                   accelerations," etc., are available.

        forcetype
            Label identifying the forces to be combined:

            STATIC - Combine the modal static forces (default).

            TOTAL - Combine the modal static plus inertial forces.

        Notes
        -----
        The SIGNIF value set with this command (including the default value of
        0.001) overrides the SIGNIF value set with the MXPAND command.

        This command is also valid for PREP7.
        """
        command = f"GRP,{signif},{label},{forcetype}"
        return self.run(command, **kwargs)

    def mmass(self, option="", zpa="", **kwargs):
        """Specifies the missing mass response calculation.

        APDL Command: MMASS

        Parameters
        ----------
        option
            Flag to activate or deactivate missing mass response calculation.

            0 (OFF or NO) - Deactivate (default).

            1 (ON or YES) - Activate.

        zpa
            Zero Period Acceleration Value. If a scale factor FACT is defined
            on the SVTYP command, it is applied to this value.

        Notes
        -----
        The missing mass calculation is valid only for single point excitation
        response spectrum analysis (SPOPT, SPRS) and for multiple point
        response spectrum analysis (SPOPT, MPRS) performed with base excitation
        using acceleration response spectrum loading. Missing mass is supported
        in a spectrum analysis only when the preceding modal analysis is
        performed with the Block Lanczos, PCG Lanczos, Supernode, or Subspace
        eigensolver (Method =LANB, LANPCG, SNODE, or SUBSP on the MODOPT
        command).

        The velocity solution is not available (Label = VELO on the combination
        command: SRSS, CQC...) when the missing mass calculation is activated.

        The missing mass calculation is not supported when the spectrum
        analysis is based on a linear perturbation modal analysis performed
        after a nonlinear base analysis.

        The missing mass is not supported when superelements are present.

        To take into account the contribution of the truncated modes, the
        residual vector (RESVEC) can be used in place of the missing mass
        response. This is of particular interest if the velocity solution is
        requested or if a nonlinear prestress is included in the analysis
        (linear perturbation), or if a superelement is present, since the
        missing mass cannot be used in these cases.

        In a multiple point response spectrum analysis (SPOPT,MPRS), the MMASS
        command must precede the participation factor calculation command
        (PFACT).

        This command is also valid in PREP7.
        """
        command = f"MMASS,{option},{zpa}"
        return self.run(command, **kwargs)

    def nrlsum(self, signif="", label="", labelcsm="", forcetype="", **kwargs):
        """Specifies the Naval Research Laboratory (NRL) sum mode combination

        APDL Command: NRLSUM
        method.

        Parameters
        ----------
        signif
            Combine only those modes whose significance level exceeds the
            SIGNIF threshold. For single point, multipoint, or DDAM response
            (SPOPT,SPRS, MPRS or DDAM), the significance level of a mode is
            defined as the mode coefficient of the mode, divided by the maximum
            mode coefficient of all modes. Any mode whose significance level is
            less than SIGNIF is considered insignificant and is not contributed
            to the mode combinations. The higher the SIGNIF threshold, the
            fewer the number of modes combined. SIGNIF defaults to 0.001. If
            SIGNIF is specified as 0.0, it is taken as 0.0. (This mode
            combination method is not valid for SPOPT,PSD.)

        label
            Label identifying the combined mode solution output.

            DISP - Displacement solution (default). Displacements, stresses, forces, etc., are
                   available.

            VELO - Velocity solution. Velocities, "stress velocities," "force velocities," etc.,
                   are available.

            ACEL - Acceleration solution. Accelerations, "stress accelerations," "force
                   accelerations," etc., are available.

        labelcsm
            Label identifying the CSM (Closely Spaced Modes) method.

            CSM - Use the CSM method.

            Blank  - Do not use the CSM method (default).

        forcetype
            Label identifying the forces to be combined:

            STATIC - Combine the modal static forces (default).

            TOTAL - Combine the modal static plus inertial forces.

        Notes
        -----
        This command is also valid in PREP7. This mode combination method is
        usually used for SPOPT,DDAM.

        This CSM method is only applicable in a DDAM analysis (SPOPT,DDAM). The
        CSM method combines two closely spaced modes into one mode when their
        frequencies are within 10 percent of the common mean frequency and
        their responses are opposite in sign. The contribution of these closely
        spaced modes is then included in the NRL sum as a single effective
        mode. Refer to Closely Spaced Modes (CSM) Method in the Mechanical APDL
        Theory Reference for more information.

        NRLSUM is not allowed in ANSYS Professional.
        """
        command = f"NRLSUM,{signif},{label},{labelcsm},{forcetype}"
        return self.run(command, **kwargs)

    def pfact(self, tblno="", excit="", parcor="", **kwargs):
        """Calculates participation factors for the PSD or multi-point response

        APDL Command: PFACT
        spectrum table.

        Parameters
        ----------
        tblno
            Input PSD (Power Spectral Density) table number for which
            participation factors are to be calculated.

        excit
            Label defining the location of excitation:

            BASE - Base excitation (default).

            NODE - Nodal excitation.

        parcor
            Label defining excitation type (applies only to SPOPT,PSD
            analysis).  Used only when partially correlated excitation is due
            to wave propagation or spatial correlation.  Defaults to partially
            correlated excitation as defined by COVAL and QDVAL commands.

            WAVE - Excitation defined by PSDWAV command.

            SPAT - Excitation defined by PSDSPL command.

        Notes
        -----
        Calculates the participation factors for a particular PSD or multi-
        point response spectrum table defined with the PSDVAL or SPVAL command.
        The Jobname.DB file must contain modal solution data in order for this
        command to calculate the participation factor.  There must be a PFACT
        command for each excitation spectrum. You are limited to 300
        excitations.

        This command is also valid in PREP7.
        """
        command = f"PFACT,{tblno},{excit},{parcor}"
        return self.run(command, **kwargs)

    def pivcheck(self, key="", prntcntrl="", **kwargs):
        """Controls the behavior of an analysis when a negative or zero equation solver pivot value is encountered.

        APDL Command: PIVCHECK

        Parameters
        ----------
        key
            Determines whether to stop or continue an analysis when a negative
            or zero equation solver pivot value is encountered:

            AUTO - Check for negative or zero pivot values for analyses
                   performed with the sparse and PCG solvers. When one is
                   encountered, an error or warning is issued, per various
                   criteria relating to the type of analysis being
                   solved. An error causes the analysis to stop; a warning
                   allows the analysis to continue. A negative pivot value
                   may be valid for some nonlinear and multiphysics
                   analyses (for example, electromagnetic and thermal
                   analyses); this key has no effect in these cases.

            ERROR - Check for negative or zero pivot values for analyses
                    performed with the sparse and PCG solvers. When one is
                    encountered, an error is issued, stopping the
                    analysis. A negative pivot value may be valid for some
                    nonlinear and multiphysics analyses (for example,
                    electromagnetic and thermal analyses); this key has no
                    effect in these cases.

            WARN - Check for negative or zero pivot values for analyses
                   performed with the sparse and PCG solvers. When one is
                   encountered, a warning is issued and the analysis
                   continues. A negative pivot value may be valid for some
                   nonlinear and multiphysics analyses (for example,
                   electromagnetic and thermal analyses); this key has no
                   effect in these cases.

            OFF - Pivot values are not checked. This key causes the
                  analysis to continue in spite of a negative or zero
                  pivot value.

        prntcntrl
            Provides print options. Print output with these options will be
            sent to the default output file, not to the files created by the
            nonlinear diagnostic tools (NLDIAG).

            ONCE - Print only the maximum and minimum pivot information on
                   the first call to the sparse solver (which is the
                   default solver). This is the default behavior.

            EVERY - Print the maximum and minimum pivot information at
                    every call to the sparse solver. This option is
                    provided for nonlinear analysis diagnostics.

        Notes
        -----
        This command is valid for all analyses. In a nonlinear analysis, a
        negative pivot may be valid. In some cases, rigid body motions in
        a nonlinear analysis will be trapped by error routines checking
        infinitely large displacements (DOF limit exceeded) or
        nonconvergence status. An under-constrained model may avoid the
        pivot check, but fail with a DOF limit exceeded error.

        Machine precision may affect whether a small pivot triggers an
        error or bypasses this checking logic. You may wish to review the
        ratio of the maximum to absolute minimum pivot values. For ratios
        exceeding 12 to 14 orders of magnitude, the accuracy of the
        computed solution may be degraded by the severe ill-conditioning
        of the assembled matrix.

        Note that negative pivots corresponding to Lagrange multiplier
        based mixed u-P elements are not checked or reported by this
        command.  Negative pivots arising from the u-P element formulation
        and related analyses can occur and lead to correct solutions.

        This command is also valid in PREP7.
        """
        command = f"PIVCHECK,{key},{prntcntrl}"
        return self.run(command, **kwargs)

    def psdcom(self, signif="", comode="", forcetype="", **kwargs):
        """Specifies the power spectral density mode combination method.

        APDL Command: PSDCOM

        Parameters
        ----------
        signif
            Combine only those modes whose significance level exceeds theSIGNIF
            threshold.  For PSD response (SPOPT,PSD), the significance level is
            defined as the modal covariance matrix term, divided by the maximum
            modal covariance matrix term.  Any term whose significance level is
            less than SIGNIF is considered insignificant and is not contributed
            to the mode combinations.  The higher the SIGNIF threshold, the
            fewer the number of terms used. SIGNIF defaults to 0.0001.  If
            SIGNIF is specified as 0.0, it is taken as 0.0.

        comode
            First COMODE number of modes to be actually combined.  COMODE must
            always be less than or equal to NMODE (input quantity NMODE on the
            SPOPT command).  COMODE defaults to NMODE.  COMODE performs a
            second level of control for the first sequential COMODE number of
            modes to be combined.  It uses the significance level threshold
            indicated by SIGNIF and operates only on the significant modes.

        forcetype
            Label identifying the forces to be combined:

            STATIC - Combine the modal static forces (default).

            TOTAL - Combine the modal static plus inertial forces.

        Notes
        -----
        This command is also valid for PREP7.   This command is valid only for
        SPOPT,PSD.

        PSDCOM is not allowed in ANSYS Professional.
        """
        command = f"PSDCOM,{signif},{comode},{forcetype}"
        return self.run(command, **kwargs)

    def psdfrq(
        self,
        tblno1="",
        tblno2="",
        freq1="",
        freq2="",
        freq3="",
        freq4="",
        freq5="",
        freq6="",
        freq7="",
        **kwargs,
    ):
        """Defines the frequency points for the input spectrum tables PSDVAL vs.

        APDL Command: PSDFRQ
        PSDFRQ for PSD analysis.

        Parameters
        ----------
        tblno1
            Input table number.  When used with the COVAL or the QDVAL command,
            TBLNO1 represents the row number of this table. Up to 200 tables
            may be defined.

        tblno2
            Input table number.  TBLNO2 is used only for the COVAL or the QDVAL
            commands and represents the column number of this table.

        freq1, freq2, freq3, . . . , freq7
            Frequency points (cycles/time) for spectrum vs. frequency tables.
            FREQ1 should be greater than zero, and values must be in ascending
            order.  Log-log interpolation will be used between frequency
            points.

        Notes
        -----
        The spectrum values may be input with the PSDVAL, COVAL , or QDVAL
        commands.  A separate PSDFRQ command must be used for each table and
        cross table defined.  Frequencies must be in ascending order.

        Repeat PSDFRQ command for additional frequency points.  Values are
        added after the last nonzero frequency.  If all fields after PSDFRQ are
        blank, all input vs. frequency tables are erased.  If TBLNO1 is
        nonblank, all corresponding PSDVAL tables are erased.  If both TBLNO1
        and TBLNO2 are nonblank,  all corresponding COVAL and QDVAL tables are
        erased.

        This command is also valid in PREP7.
        """
        command = f"PSDFRQ,{tblno1},{tblno2},{freq1},{freq2},{freq3},{freq4},{freq5},{freq6},{freq7}"
        return self.run(command, **kwargs)

    def psdgraph(self, tblno1="", tblno2="", displaykey="", **kwargs):
        """Displays input PSD curves

        APDL Command: PSDGRAPH

        Parameters
        ----------
        tblno1
            PSD table number to display.

        tblno2
            Second PSD table number to display. TBLNO2 is used only in
            conjunction with the COVAL or the QDVAL commands.

        displaykey
            Key to display the points markers and numbering:

                0 - Display points markers and numbering (default).
                1 - Display points numbering only.
                2 - Display points markers only.
                3 - No points markers or numbering.

        Notes
        -----
        The input PSD tables are displayed in log-log format as dotted lines.
        The best-fit curves, used to perform the closed-form integration, are
        displayed as solid lines. If there is a significant discrepancy between
        the two, then you should add one or more intermediate points to the
        table to obtain a better fit.

        If TBLNO2 is zero, blank, or equal to TBLNO1, then the autospectra
        (PSDVAL) are displayed for TBLNO1. If TBLNO2 is also specified, then
        the autospectra for TBLNO1 and TBLNO2 are displayed, along with the
        corresponding cospectra (COVAL) and quadspectra (QDVAL), if they are
        defined.

        This command is valid in any processor.
        """
        command = f"PSDGRAPH,{tblno1},{tblno2},{displaykey}"
        return self.run(command, **kwargs)

    def psdres(self, lab="", relkey="", **kwargs):
        """Controls solution output written to the results file from a PSD

        APDL Command: PSDRES
        analysis.

        Parameters
        ----------
        lab
            Label identifying the solution output:

            DISP - Displacement solution (default).  One-sigma displacements, stresses, forces,
                   etc.  Written as load step 3 on File.RST.

            VELO - Velocity solution.  One-sigma velocities, "stress velocities," "force
                   velocities," etc.  Written as load step 4 of File.RST.

            ACEL - Acceleration solution.  One-sigma accelerations, "stress accelerations," "force
                   accelerations," etc.  Written as load step 5 on File.RST.

        relkey
            Key defining relative or absolute calculations:

            REL - Calculations are relative to the base excitation (default).

            ABS - Calculations are absolute.

            OFF - No calculation of solution output identified by Lab.

        Notes
        -----
        Controls the amount and form of solution output written to the results
        file from a PSD analysis.  One-sigma values of the relative or absolute
        displacement solution, relative or absolute velocity solution, relative
        or absolute acceleration solution, or any combination may be included
        on the results file.

        This command is also valid in PREP7.

        Distributed ANSYS Restriction: This command is not supported in
        Distributed ANSYS.
        """
        command = f"PSDRES,{lab},{relkey}"
        return self.run(command, **kwargs)

    def psdspl(self, tblno="", rmin="", rmax="", **kwargs):
        """Defines a partially correlated excitation in a PSD analysis.

        APDL Command: PSDSPL

        Parameters
        ----------
        tblno
            Input PSD table number defined with PSDVAL command.

        rmin
            Minimum distance between excitation points which are partially
            correlated. Excited nodes closer than RMIN will be fully
            correlated.

        rmax
            Maximum distance between excitation points which are partially
            correlated. Excited nodes farther apart than RMAX will be
            uncorrelated.

        Notes
        -----
        Defines a partially correlated excitation in terms of a sphere of
        influence relating excitation point geometry (in a PSD analysis).  If
        the distance between any two excitation points is less than RMIN, then
        the excitation is fully correlated.  If the distance is greater than
        RMAX, then the excitation is uncorrelated.  If the distance lies
        between RMIN and RMAX, then the excitation is partially correlated with
        the degree of correlation dependent on the separation distance between
        the points. This command is not available for a pressure PSD analysis.

        This command is also valid in PREP7.
        """
        command = f"PSDSPL,{tblno},{rmin},{rmax}"
        return self.run(command, **kwargs)

    def psdunit(self, tblno="", type_="", gvalue="", **kwargs):
        """Defines the type of input PSD.

        APDL Command: PSDUNIT

        Parameters
        ----------
        tblno
            Input table number.

        type\\_
            Label identifying the type of spectrum:

            DISP - Displacement spectrum (in terms of displacement2/Hz ).

            VELO - Velocity spectrum (in terms of velocity2/Hz ).

            ACEL - Acceleration spectrum (in terms of acceleration2/Hz ).

            ACCG - Acceleration spectrum (in terms of g2/Hz ).

            FORC - Force spectrum (in terms of force2/Hz ).

            PRES - Pressure spectrum (in terms of pressure2/Hz ).

        gvalue
            Value of acceleration due to gravity in any arbitrary units for
            Type=ACCG.  Default is 386.4 in/sec2.

        Notes
        -----
        Defines the type of PSD defined by the PSDVAL, COVAL, and QDVAL
        commands.

        Force (FORC) and pressure (PRES) type spectra can be used only as a
        nodal excitation.

        GVALUE is valid only when type ACCG is specified.  A zero or negative
        value cannot be used.  A parameter substitution can also be performed.

        This command is also valid in PREP7.
        """
        command = f"PSDUNIT,{tblno},{type_},{gvalue}"
        return self.run(command, **kwargs)

    def psdval(
        self,
        tblno="",
        sv1="",
        sv2="",
        sv3="",
        sv4="",
        sv5="",
        sv6="",
        sv7="",
        **kwargs,
    ):
        """Defines PSD values.

        APDL Command: PSDVAL

        Parameters
        ----------
        tblno
            Input table number being defined.

        sv1, sv2, sv3, . . . , sv7
            Spectral values corresponding to the frequency points [PSDFRQ].
            Values are interpreted as defined with the PSDUNIT command.

        Notes
        -----
        Defines PSD values to be associated with the previously defined
        frequency points.

        Repeat PSDVAL command for additional values, up to the number of
        frequency points [PSDFRQ].  Values are added after the last nonzero
        value.

        This command is also valid in PREP7.
        """
        command = f"PSDVAL,{tblno},{sv1},{sv2},{sv3},{sv4},{sv5},{sv6},{sv7}"
        return self.run(command, **kwargs)

    def psdwav(self, tblno="", vx="", vy="", vz="", **kwargs):
        """Defines a wave propagation excitation in a PSD analysis.

        APDL Command: PSDWAV

        Parameters
        ----------
        tblno
            Input PSD table number defined with PSDVAL command.

        vx
            Global Cartesian X-velocity of traveling wave.

        vy
            Global Cartesian Y-velocity of traveling wave.

        vz
            Global Cartesian Z-velocity of traveling wave.

        Notes
        -----
        Defines a traveling wave in a PSD analysis. This command is not
        available for a pressure PSD analysis.

        This command is also valid in PREP7.
        """
        command = f"PSDWAV,{tblno},{vx},{vy},{vz}"
        return self.run(command, **kwargs)

    def qdval(
        self,
        tblno1="",
        tblno2="",
        sv1="",
        sv2="",
        sv3="",
        sv4="",
        sv5="",
        sv6="",
        sv7="",
        **kwargs,
    ):
        """Defines PSD quadspectral values.

        APDL Command: QDVAL

        Parameters
        ----------
        tblno1
            First input PSD table number associated with this spectrum.

        tblno2
            Second input PSD table number associated with this spectrum.

        sv1, sv2, sv3, . . . , sv7
            PSD quadspectral values corresponding to the frequency points
            [PSDFRQ].

        Notes
        -----
        Defines PSD quadspectral values to be associated with the previously
        defined frequency points.  Repeat QDVAL command with the same table
        number for additional points.  Unlike autospectra [PSDVAL], the
        quadspectra can be positive or negative.  The quadspectral curve
        segment where there is a sign change is interpolated linearly (the rest
        of the curve segments use log-log interpolation).  For better accuracy,
        choose as small a curve segment as possible wherever a sign change
        occurs.

        Two table numbers are required since values are off-diagonal terms.
        This command is valid for SPOPT,PSD only.

        This command is also valid in PREP7.

        Distributed ANSYS Restriction: This command is not supported in
        Distributed ANSYS.
        """
        command = f"QDVAL,{tblno1},{tblno2},{sv1},{sv2},{sv3},{sv4},{sv5},{sv6},{sv7}"
        return self.run(command, **kwargs)

    def rock(self, cgx="", cgy="", cgz="", omx="", omy="", omz="", **kwargs):
        """Specifies a rocking response spectrum.

        APDL Command: ROCK

        Parameters
        ----------
        cgx, cgy, cgz
            Global Cartesian X, Y, and Z location of center of rotation about
            which rocking occurs.

        omx, omy, omz
            Global Cartesian angular velocity components associated with the
            rocking.

        Notes
        -----
        Specifies a rocking response spectrum effect in the spectrum
        (ANTYPE,SPECTR) analysis.

        The excitation direction with rocking included is not normalized to
        one; rather, it scales the spectrum. For more information, see
        Participation Factors and Mode Coefficients.

        This command is also valid in PREP7.
        """
        command = f"ROCK,{cgx},{cgy},{cgz},{omx},{omy},{omz}"
        return self.run(command, **kwargs)

    def rose(self, signif="", label="", td="", forcetype="", **kwargs):
        """Specifies the Rosenblueth mode combination method.

        APDL Command: ROSE

        Parameters
        ----------
        signif
            Combine only those modes whose significance level exceeds the
            SIGNIF threshold. For single point, multipoint, or DDAM response
            (SPOPT, SPRS, MPRS, or DDAM), the significance level of a mode is
            defined as the mode coefficient of the mode, divided by the maximum
            mode coefficient of all modes. Any mode whose significance level is
            less than SIGNIF is considered insignificant and does not
            contribute to the mode combinations. The higher the SIGNIF
            threshold, the fewer the number of modes combined. SIGNIF defaults
            to 0.001. If SIGNIF is specified as 0.0, it is taken as 0.0.

        label
            Label identifying the combined mode solution output.

            DISP - Displacement solution (default). Displacements, stresses, forces, etc., are
                   available.

            VELO - Velocity solution. Velocities, "stress velocities," "force velocities," etc.,
                   are available.

            ACEL - Acceleration solution. Accelerations, "stress accelerations," "force
                   accelerations," etc. are available.

        td
            Time duration for earthquake or shock spectrum. TD defaults to 10.

        forcetype
            Label identifying the forces to be combined:

            STATIC - Combine the modal static forces (default).

            TOTAL - Combine the modal static plus inertial forces.

        Notes
        -----
        For more information on spectrum analysis combination methods, see
        Combination of Modes

        This command is also valid in PREP7.

        Distributed ANSYS Restriction: This command is not supported in
        Distributed ANSYS.
        """
        command = f"ROSE,{signif},{label},{td},{forcetype}"
        return self.run(command, **kwargs)

    def rigresp(self, option="", method="", val1="", val2="", **kwargs):
        """Specifies the rigid response calculation.

        APDL Command: RIGRESP

        Parameters
        ----------
        option
            Flag to activate or deactivate the rigid response calculation:

            1 (ON or YES) - Activate.

            2 (OFF or NO) - Deactivate. This value is the default.

        method
            Method used to calculate the rigid response:

            GUPTA - Gupta method.

            LINDLEY - Lindley-Yow method.

        val1
            If Method = GUPTA, Val1 represents the frequency F1 in Hertz.

        val2
            If Method = GUPTA, Val2 represents the frequency F2 in Hertz.

        Notes
        -----
        This rigid response calculation is only valid for single point response
        spectrum analysis (SPOPT, SPRS) and multiple point response spectrum
        analysis (SPOPT, MPRS) with combination methods (SRSS), complete
        quadratic (CQC) or Rosenblueth (ROSE)

        This command is also valid in PREP7.

        Only Sptype = SPRS is allowed in ANSYS Professional.
        """
        command = f"RIGRESP,{option},{method},{val1},{val2}"
        return self.run(command, **kwargs)

    def sed(self, sedx="", sedy="", sedz="", cname="", **kwargs):
        """Defines the excitation direction for response spectrum and PSD

        APDL Command: SED
        analyses.

        Parameters
        ----------
        sedx, sedy, sedz
            Global Cartesian coordinates of a point that defines a line
            (through the origin) corresponding to the excitation direction.
            For example: 0.0, 1.0, 0.0 defines global Y as the spectrum
            direction.

        cname
            The component name corresponding to the group of excited nodes.
            Only applies to base excitation multi-point response spectrum
            analysis (SPOPT, MPRS) and power spectral density analysis (SPOPT,
            PSD). Defaults to no component.

        Notes
        -----
        In single-point response spectrum analysis (SPOPT,SPRS), the excitation
        direction without rocking (ROCK) is normalized to one so that the SEDX,
        SEDY, and SEDZ values do not scale the spectrum.  The excitation
        direction with rocking is not normalized. The SEDX, SEDY, and SEDZ
        values must be consistent with the OMX, OMY, and OMZ values on the ROCK
        command. The calculated direction then scales the spectrum. For more
        information, see Participation Factors and Mode Coefficients.

        In multi-point response spectrum analysis (SPOPT,MPRS) and power
        spectral density analysis (SPOPT,PSD), the excitation direction is
        normalized to one so that the SEDX, SEDY, and SEDZ values do not scale
        the spectrum.  The component name (Cname) is required. The constraints
        corresponding to the excitation direction are applied to the component
        nodes.

        This command is also valid in PREP7.
        """
        command = f"SED,{sedx},{sedy},{sedz},{cname}"
        return self.run(command, **kwargs)

    def spdamp(self, tblno="", curvno="", dampratio="", **kwargs):
        """Defines input spectrum damping in a multi-point response spectrum

        APDL Command: SPDAMP
        analysis.

        Parameters
        ----------
        tblno
            Input table number. Corresponds to the frequency table number
            (TBLNO on the SPFREQ command).

        curvno
            Input curve number. Corresponds to the spectrum values curve number
            (CURVNO on the SPVAL command).

        dampratio
            Damping ratio for the response spectrum curve. Up to 20 different
            curves may be defined, each with a different damping ratio. Damping
            values must be input in ascending order.

        Notes
        -----
        Defines multi-point response spectrum damping value to be associated
        with:

        Previously defined frequency points (SPFREQ).

        Subsequently defined spectrum points (SPVAL).

        Damping values are used only to identify input spectrum values for the
        mode coefficients calculation.

        The curve number must be input in ascending order starting with 1.

        This command is also valid in PREP7.

        Distributed ANSYS Restriction: This command is not supported in
        Distributed ANSYS.
        """
        command = f"SPDAMP,{tblno},{curvno},{dampratio}"
        return self.run(command, **kwargs)

    def spfreq(
        self,
        tblno="",
        freq1="",
        freq2="",
        freq3="",
        freq4="",
        freq5="",
        freq6="",
        freq7="",
        **kwargs,
    ):
        """Defines the frequency points for the input spectrum tables SPVAL vs.

        APDL Command: SPFREQ
        SPFREQ for multi-point spectrum analysis.

        Parameters
        ----------
        tblno
            Input table number. Up to 200 tables may be defined.

        freq1, freq2, freq3,..., freq7
            Frequency points (Hz) for spectrum vs. frequency tables. FREQ1
            should be greater than zero, and values must be in ascending order.

        Notes
        -----
        The spectrum values are input with the SPVAL command. A separate SPFREQ
        command must be used for each table defined. Frequencies must be in
        ascending order.

        Repeat SPFREQ command for additional frequency points. Values are added
        after the last nonzero frequency.

        If all fields after SPFREQ are blank, all input vs. frequency tables
        are erased. If TBLNO is the only non-blank field, all corresponding
        SPVAL curves are erased.

        Use the SPTOPT and STAT commands to list current frequency points.

        This command is also valid in PREP7.

        Distributed ANSYS Restriction: This command is not supported in
        Distributed ANSYS.
        """
        command = (
            f"SPFREQ,{tblno},{freq1},{freq2},{freq3},{freq4},{freq5},{freq6},{freq7}"
        )
        return self.run(command, **kwargs)

    def spgraph(self, tblno="", curvno="", curvnobeg="", **kwargs):
        """Displays input spectrum curves for MPRS analysis.

        APDL Command: SPGRAPH

        Parameters
        ----------
        tblno
            Table number to display.  Defaults to 1.

        curvno
            Curve number to display. Defaults to none.

        curvnobeg
            Beginning of the curve number range to display. Defaults to 1.

        Notes
        -----
        You can display up to 10 input spectrum curves (SPVAL and SPFREQ
        commands) with log X scale.

        If the input spectrum curves are not associated with a damping value
        (SPDAMP command), CURVNO and CURVNOBeg are not applicable and table
        TBLNO is displayed. Otherwise, specify CURVNO or CURVNOBeg:

        if CURVNO is used, one curve is displayed.

        if CURVNOBeg is used, up to 10 curves are displayed. CURVNOBeg is the
        beginning of the curve number range of interest.
        """
        command = f"SPGRAPH,{tblno},{curvno},{curvnobeg}"
        return self.run(command, **kwargs)

    def spopt(self, sptype="", nmode="", elcalc="", modereusekey="", **kwargs):
        """Selects the spectrum type and other spectrum options.

        APDL Command: SPOPT

        Parameters
        ----------
        sptype
            Spectrum type:

            SPRS - Single point excitation response spectrum (default).  See also the SVTYP
                   command.

            MPRS - Multiple point excitation response spectrum.

            DDAM - Dynamic design analysis method.

            PSD - Power spectral density.

        nmode
            Use the first NMODE modes from the modal analysis.  Defaults to all
            extracted modes, as specified by the MODOPT and BUCOPT commands.
            NMODE cannot be larger than 10000.

        elcalc
            Element results calculation key (for Sptype = PSD only):

            NO - Do not calculate element results and reaction forces (default).

            YES - Calculate element results and reaction forces, as well as the nodal degree of
                  freedom solution.

        modereusekey
            Key for existing MODE file reuse when running multiple spectrum
            analyses:

            NO - No spectrum analysis has been performed yet (default).

            YES - This is not the first spectrum analysis.  The MODE file will be reused and the
                  necessary files will be cleaned up for the new spectrum
                  analysis.

        Notes
        -----
        Valid only for a spectrum analysis (ANTYPE,SPECTR).  This operation
        must be preceded by a modal solution (ANTYPE,MODAL) with the
        appropriate files available.  Both the spectrum analysis and the
        preceding modal analysis must be performed under the same ANSYS version
        number.

        If used in SOLUTION, this command is valid only within the first load
        step.

        This command is also valid in PREP7.

        Only Sptype = SPRS is allowed in ANSYS Professional.
        """
        command = f"SPOPT,{sptype},{nmode},{elcalc},{modereusekey}"
        return self.run(command, **kwargs)

    def spunit(self, tblno="", type_="", gvalue="", keyinterp="", **kwargs):
        """Defines the type of multi-point response spectrum.

        APDL Command: SPUNIT

        Parameters
        ----------
        tblno
            Input table number.

        type\\_
            Label identifying the type of spectrum:

            DISP   - Displacement spectrum (SPVAL values interpreted as displacements with units of
                     length).

            VELO   - Velocity spectrum (SPVAL values interpreted as velocities with units of
                     length/time).

            ACEL   - Acceleration spectrum (SPVAL values interpreted as accelerations with units of
                     length/time2).

            ACCG   - Acceleration spectrum (SPVAL values interpreted as accelerations with units of
                     g/time2).

            FORC   - Force spectrum.

            PRES   - Pressure spectrum.

        gvalue
            Value of acceleration due to gravity in any arbitrary units for
            Type=ACCG table. Default is 386.4 in/sec2.

        keyinterp
            Key to activate or deactivate the linear interpolation between
            input response spectrum points and input response spectrum curves:

            0 (OFF or NO) - Deactivate linear and use logarithmic interpolation. This value is the default.

            1 (ON or YES) - Activate linear interpolation.

        Notes
        -----
        Defines the type of multi-point response spectrum defined by the SPFREQ
        and  SPVAL commands.

        Force (FORC) and pressure (PRES) type spectra can be used only as a
        nodal excitation.

        GVALUE is valid only when type=ACCG is specified. A zero or negative
        value cannot be used. A parameter substitution can also be performed.

        This command is also valid in PREP7.
        """
        command = f"SPUNIT,{tblno},{type_},{gvalue},{keyinterp}"
        return self.run(command, **kwargs)

    def spval(
        self,
        tblno="",
        curvno="",
        sv1="",
        sv2="",
        sv3="",
        sv4="",
        sv5="",
        sv6="",
        sv7="",
        **kwargs,
    ):
        """Defines multi-point response spectrum values.

        APDL Command: SPVAL

        Parameters
        ----------
        tblno
            Input table number. It corresponds to TBLNO on the SPFREQ command.

        curvno
            Input curve number. It corresponds to CURVNO on the SPDAMP command
            (optional).

        sv1, sv2, sv3, , , . . . , sv7
            Spectral values corresponding to the frequency points (SPFREQ) and
            damping ratio (SPDAMP). Values are interpreted as defined with the
            SPUNIT command.

        Notes
        -----
        Defines multi-point response spectrum values to be associated with the
        previously defined frequency points (SPFREQ). It can also be associated
        with the previously defined damping value (SPDAMP). If CURVNO is not
        specified, the input spectrum is not associated with a damping value.

        Repeat SPVAL command for additional values, up to the number of
        frequency points (SPFREQ). Values are added after the last nonzero
        value.

        The interpolation method between response spectrum points and curves is
        specified using KeyInterp on the SPUNIT command. It is logarithmic by
        default.

        Use the SPTOPT and STAT commands to list current spectrum curve values.

        This command is also valid in PREP7.
        """
        command = f"SPVAL,{tblno},{curvno},{sv1},{sv2},{sv3},{sv4},{sv5},{sv6},{sv7}"
        return self.run(command, **kwargs)

    def srss(self, signif="", label="", abssumkey="", forcetype="", **kwargs):
        """Specifies the square root of sum of squares mode combination method.

        APDL Command: SRSS

        Parameters
        ----------
        signif
            Combine only those modes whose significance level exceeds the
            SIGNIF threshold.  For single point, multipoint, or DDAM response
            (SPOPT,SPRS, MPRS or DDAM), the significance level of a mode is
            defined as the mode coefficient of the mode, divided by the maximum
            mode coefficient of all modes.  Any mode whose significance level
            is less than SIGNIF is considered insignificant and is not
            contributed to the mode combinations.  The higher the SIGNIF
            threshold, the fewer the number of modes combined. SIGNIF defaults
            to 0.001.  If SIGNIF is specified as 0.0, it is taken as 0.0.
            (This mode combination method is not valid for SPOPT,PSD.)

        label
            Label identifying the combined mode solution output.

            DISP - Displacement solution (default).  Displacements, stresses, forces, etc., are
                   available.

            VELO - Velocity solution.  Velocities, "stress velocities," "force velocities," etc.,
                   are available.

            ACEL - Acceleration solution.  Accelerations, "stress accelerations," "force
                   accelerations," etc., are available.

        abssumkey
            Absolute Sum combination key (for SPOPT,MPRS only):

            NO - Do not use the Absolute Sum method (default).

            YES - Combine the modes per excitation direction using the Absolute Sum method, then
                  combine the resulting quantities using the square root of sum
                  of squares method.

        forcetype
            Label identifying the forces to be combined:

            STATIC - Combine the modal static forces (default).

            TOTAL - Combine the modal static plus inertial forces.

        Notes
        -----
        This command is also valid for PREP7.
        """
        command = f"SRSS,{signif},{label},{abssumkey},{forcetype}"
        return self.run(command, **kwargs)

    def sv(
        self,
        damp="",
        sv1="",
        sv2="",
        sv3="",
        sv4="",
        sv5="",
        sv6="",
        sv7="",
        sv8="",
        sv9="",
        **kwargs,
    ):
        """Defines spectrum values to be associated with frequency points.

        APDL Command: SV

        Parameters
        ----------
        damp
            Damping ratio for this response spectrum curve.  If the same as a
            previously defined curve, the SV values are added to the previous
            curve.  Up to four different curves may be defined, each with a
            different damping ratio.  Damping values must be input in ascending
            order.

        sv1, sv2, sv3, . . . , sv9
            Spectrum values corresponding to the frequency points [FREQ].
            Values are interpreted as defined with the SVTYP command.   SV
            values should not be zero.  Values required outside the frequency
            range use the extreme input values.

        Notes
        -----
        Defines the spectrum values to be associated with the previously
        defined frequency points [FREQ].  Applies only to the single-point
        response spectrum.  Damping has no effect on the frequency solution.
        Damping values are used only to identify SV curves for the mode
        combinations calculation.  Only the curve with the lowest damping value
        is used in the initial mode coefficient calculation.  Use STAT command
        to list current spectrum curve values.

        Repeat SV command for additional SV points (100 maximum per DAMP
        curve).  SV values are added to the DAMP curve after the last nonzero
        SV value.

        The interpolation method between response spectrum points and curves is
        specified using KeyInterp in the SVTYP command. It is logarithmic by
        default.

        This command is also valid in PREP7.
        """
        command = f"SV,{damp},{sv1},{sv2},{sv3},{sv4},{sv5},{sv6},{sv7},{sv8},{sv9}"
        return self.run(command, **kwargs)

    def svplot(self, optionscale="", damp1="", damp2="", damp3="", damp4="", **kwargs):
        """Displays input spectrum curves.

        APDL Command: SVPLOT

        Parameters
        ----------
        optionscale
            Flag to activate or deactivate input spectrum value scaling:

            OFF  - Do not scale the input spectrum values with scale factor FACT (SVTYP command).
                   This is the default value.

            ON  - Scale the input spectrum values with scale factor FACT (SVTYP command)

        damp1
            Damping ratio corresponding to DAMP (SV command) defining the first
            spectrum curve.

        damp2
            Damping ratio corresponding to DAMP (SV command) defining the
            second spectrum curve.

        damp3
            Damping ratio corresponding to DAMP (SV command) defining the third
            spectrum curve.

        damp4
            Damping ratio corresponding to DAMP (SV command) defining the
            fourth spectrum curve.

        Notes
        -----
        You can display up to four input spectrum tables (SV and FREQ commands)
        with log X scale. If no damping ratio is specified, all spectrum tables
        are displayed.

        This command is valid in any processor.
        """
        command = f"SVPLOT,{optionscale},{damp1},{damp2},{damp3},{damp4}"
        return self.run(command, **kwargs)

    def svtyp(self, ksv="", fact="", keyinterp="", **kwargs):
        """Defines the type of single-point response spectrum.

        APDL Command: SVTYP

        Parameters
        ----------
        ksv
            Response spectrum type:

            0 - Seismic velocity response spectrum loading (SV values interpreted as velocities
                with units of length/time).

            1 - Force response spectrum loading (SV values interpreted as force amplitude
                multipliers).

            2 - Seismic acceleration response spectrum loading (SV values interpreted as
                accelerations with units of length/time2).

            3 - Seismic displacement response spectrum loading (SV values interpreted as
                displacements with units of length).

            4 - PSD loading (SV values interpreted as acceleration2/(cycles/time), such as
                (in/sec2)2/Hz (not g2/Hz)).  (Not recommended)

        fact
            Scale factor applied to spectrum values (defaults to 1.0).  Values
            are scaled when the solution is initiated [SOLVE].  Database values
            remain the same.

        keyinterp
            Key to activate or deactivate the linear interpolation between
            input response spectrum points and input response spectrum curves:

            0 (OFF or NO) - Deactivate linear and use logarithmic interpolation. This value is the default.

            1 (ON or YES) - Activate linear interpolation.

        Notes
        -----
        Defines the type of single-point response spectrum [SPOPT].  The
        seismic excitation direction is defined with the SED command.

        This command is also valid in PREP7.
        """
        command = f"SVTYP,{ksv},{fact},{keyinterp}"
        return self.run(command, **kwargs)

    def vddam(self, vf="", va="", vb="", vc="", **kwargs):
        """Specifies the velocity spectrum computation constants for the analysis

        APDL Command: VDDAM
        of shock resistance of shipboard structures.

        Parameters
        ----------
        vf
            Direction-dependent velocity coefficient for elastic or elastic-
            plastic analysis option (Default = 0).

        va, vb, vc
            Coefficients for the DDAM velocity spectrum equations.  See Dynamic
            Design Analysis Method in the Mechanical APDL Theory Reference.
            Default for these coefficients is zero.

        Notes
        -----
        This command specifies velocity coefficients to analyze shock
        resistance of shipboard equipment.  These coefficients are used to
        compute mode coefficients according to the equations given in Dynamic
        Design Analysis Method in the Mechanical APDL Theory Reference.  The
        form of these equations is based on the Naval NRL Dynamic Design
        Analysis Method.  This command, along with the ADDAM and SED commands,
        is used with the spectrum (ANTYPE,SPECTR) analysis as a special purpose
        alternative to the SV, FREQ, and SVTYP commands.  The mass and length
        units of the model must be in pounds and inches, respectively.

        DDASPEC may alternatively be used to calculate spectrum coefficients.

        This command is also valid in PREP7.
        """
        command = f"VDDAM,{vf},{va},{vb},{vc}"
        return self.run(command, **kwargs)



# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class HardPoints:
    def hptcreate(
        self,
        type_="",
        entity="",
        nhp="",
        label="",
        val1="",
        val2="",
        val3="",
        **kwargs,
    ):
        """Defines a hard point.

        APDL Command: HPTCREATE

        Parameters
        ----------
        type\\_
            Type of entity on which the hard point will be created.

            LINE - Hard point will be created on a line.

            AREA - Hard point will be created within an area (not on the boundaries).

        entity
            Number of the line or area on which the hard point will be created.

        nhp
            Number assigned to the hard point.  Defaults to the lowest
            available hard point number.

        label
            If LABEL = COORD, VAL1, VAL2, and VAL3 are the respective global X,
            Y, and Z coordinates.  If LABEL = RATIO, VAL1 is the parameter
            value (this is available only for lines).  Valid parameter values
            are between 0 and 1.  VAL2 and VAL3 are ignored.

        val1
            If LABEL = RATIO, ratio value for line.  If LABEL = COORD, global X
            coordinate value.

        val2
            If LABEL = COORD, global Y coordinate value.

        val3
            If LABEL = COORD, global Z coordinate value.

        Notes
        -----
        The ability to enter a parameter value provides a simple way of
        positioning hard points on lines.  For example, to place a hard point
        halfway along a line, one can simply specify a VAL1 value of 0.5.

        For models imported through the DEFAULT IGES filter, you can place hard
        points on models only by specifying coordinates (you can't place a hard
        point using interactive picking).

        If you issue any commands that update the geometry of an entity, such
        as Boolean or simplification commands, any hard points associated with
        that entity are deleted.  Therefore, you should add any hard points
        after completing the solid model.  If you delete an entity that has
        associated hard points, those hard points are either

        Deleted along with the entity (if the hard point is not associated with
        any other entities).

        Detached from the deleted entity (if the hard point is associated with
        additional entities).

        When archiving your model (CDWRITE), hardpoint information cannot be
        written to the IGES file. The Jobname.cdb file can be written with the
        CDWRITE,DB option.

        Hard points are only applicable for area and volume meshing, not for
        beams.
        """
        command = f"HPTCREATE,{type_},{entity},{nhp},{label},{val1},{val2},{val3}"
        return self.run(command, **kwargs)

    def hptdelete(self, np1="", np2="", ninc="", **kwargs):
        """Deletes selected hardpoints.

        APDL Command: HPTDELETE

        Parameters
        ----------
        np1, np2, ninc
            Delete the pattern of hard points beginning with NP1 to NP2 in
            steps of NINC (defaults to 1).  If NP1 = ALL, NP2 and NINC are
            ignored and the pattern is all selected hard points [KSEL].  If NP1
            = P, graphical picking is enabled and all remaining command fields
            are ignored.

        Notes
        -----
        Deletes all attributes attached to the designated hard points as well
        as the hard points themselves.  If any entity is attached to a
        designated hard point, the command detaches the hard point from that
        entity (the program will alert you that this will occur).
        """
        command = f"HPTDELETE,{np1},{np2},{ninc}"
        return self.run(command, **kwargs)



# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class BinDump:
    def dump(self, nstrt="", nstop="", **kwargs):
        """Dumps the contents of a binary file.

        APDL Command: DUMP

        Parameters
        ----------
        nstrt, nstop
            Dump file from record NSTRT (defaults to 1) to NSTOP (defaults to
            NSTRT).  If NSTRT = HEAD, dump only record 1 of the file (NSTOP and
            the format specification are ignored).  If NSTRT = ALL, dump the
            entire file.

        Notes
        -----
        Dumps the file named on the AUX2 FILEAUX2 command according the format
        specified on the FORM command.
        """
        command = f"DUMP,{nstrt},{nstop}"
        return self.run(command, **kwargs)

    def fileaux2(self, fname="", ident="", **kwargs):
        """Specifies the binary file to be dumped.

        APDL Command: FILEAUX2

        Parameters
        ----------
        fname
            File name and directory path (248 characters maximum,
            including the characters needed for the directory path).
            An unspecified directory path defaults to the working
            directory; in this case, you can use all 248 characters
            for the file name.

        ident
            ANSYS filename identifier.  See the Basic Analysis Guide
            for file descriptions and identifiers.  If not an ANSYS
            identifier, Ident will be used as the filename extension.

        Notes
        -----
        Specifies the binary file to be dumped with the DUMP command.
        """
        command = f"FILEAUX2,{fname},{ident}"
        return self.run(command, **kwargs)

    def form(self, lab="", **kwargs):
        """Specifies the format of the file dump.

        APDL Command: FORM

        Parameters
        ----------
        lab
            Format:

            RECO
                Basic record description only (minimum output) (default).

            TEN
                Same as RECO plus the first ten words of each record.

            LONG
                Same as RECO plus all words of each record.

        Notes
        -----
        Specifies the format of the file dump (from the DUMP command).
        """
        command = f"FORM,{lab}"
        return self.run(command, **kwargs)

    def ptr(self, loc="", base="", **kwargs):
        """Dumps the record of a binary file.

        APDL Command: PTR

        Parameters
        ----------
        loc, base
            Dump the file record starting at pointer LOC. BASE is the base
            pointer, and would be used if LOC is a relative pointer.

        Notes
        -----
        Dumps the record of the file named on the AUX2 FILEAUX2 command
        according the format specified on the FORM command.
        """
        command = f"PTR,{loc},{base}"
        return self.run(command, **kwargs)



# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class ArrayParam:
    def mfouri(self, oper="", coeff="", mode="", isym="", theta="", curve="", **kwargs):
        """Calculates the coefficients for, or evaluates, a Fourier series.

        APDL Command: ``*MFOURI``

        Parameters
        ----------
        oper
            Type of Fourier operation:

            FIT
              Calculate Fourier coefficients COEFF from MODE, ISYM,
              THETA, and CURVE.

            EVAL
              Evaluate the Fourier curve CURVE from
              COEFF, MODE, ISYM and THETA.

        coeff
            Name of the array parameter vector containing the Fourier
            coefficients (calculated if Oper = FIT, required as input if Oper =
            EVAL).  See ``*SET`` for name restrictions.

        mode
            Name of the array parameter vector containing the mode numbers of
            the desired Fourier terms.

        isym
            Name of the array parameter vector containing the symmetry key for
            the corresponding Fourier terms.  The vector should contain keys
            for each term as follows:

            0 or 1
              Symmetric (cosine) term

            -1
              Antisymmetric (sine) term.

        theta, curve
            Names of the array parameter vectors containing the theta vs. curve
            description, respectively.  Theta values should be input in
            degrees.  If Oper = FIT, one curve value should be supplied with
            each theta value.  If Oper = EVAL, one curve value will be
            calculated for each theta value.

        Notes
        -----
        Calculates the coefficients of a Fourier series for a given
        curve, or evaluates the Fourier curve from the given (or
        previously calculated) coefficients.  The lengths of the
        COEFF, MODE, and ISYM vectors must be the same--typically two
        times the number of modes desired, since two terms (sine and
        cosine) are generally required for each mode.  The lengths of
        the CURVE and THETA vectors should be the same or the smaller
        of the two will be used.  There should be a sufficient number
        of points to adequately define the curve--at least two times
        the number of coefficients.  A starting array element number
        (1) must be defined for each array parameter vector.

        The vector specifications ``*VLEN``, ``*VCOL``, ``*VABS``,
        ``*VFACT``, and ``*VCUM`` do not apply to this command.  Array
        elements should not be skipped with the ``*VMASK`` and the
        NINC value of the ``*VLEN`` specifications.  The vector being
        calculated (COEFF if Oper is FIT, or CURVE if Oper is EVAL)
        must exist as a dimensioned array [``*DIM``].

        This command is valid in any processor.
        """
        command = f"*MFOURI,{oper},{coeff},{mode},{isym},{theta},{curve}"
        return self.run(command, **kwargs)

    def mfun(self, parr="", func="", par1="", **kwargs):
        """Copies or transposes an array parameter matrix.

        APDL Command: ``*MFUN``

        Parameters
        ----------
        parr
            The name of the resulting array parameter matrix.  See ``*SET`` for
            name restrictions.

        func
            Copy or transpose function:

            Par1 is copied to ParR - Par1 is transposed to ParR.  Rows
            (m) and columns (n) of Par1 matrix are transposed to
            resulting ParR matrix of shape (n,m).

        par1
            Array parameter matrix input to the operation.

        Notes
        -----
        Operates on one input array parameter matrix and produces one output
        array parameter matrix according to:

        ParR = f(Par1)

        where the function (f) is either a copy or transpose, as described
        above.

        Functions are based on the standard FORTRAN definitions where
        possible.  ParR may be the same as Par1.  Starting array
        element numbers must be defined for each array parameter
        matrix if it does not start at the first location. For
        example, ``*MFUN,A(1,5),COPY,B(2,3)`` copies matrix B
        (starting at element (2,3)) to matrix A (starting at element
        (1,5)).  The diagonal corner elements for each submatrix must
        be defined: the upper left corner by the array starting
        element (on this command), the lower right corner by the
        current values from the ``*VCOL`` and ``*VLEN`` commands.  The
        default values are the (1,1) element and the last element in
        the matrix.  No operations progress across matrix planes (in
        the 3rd dimension).  Absolute values and scale factors may be
        applied to all parameters [``*VABS``, ``*VFACT``].  Results
        may be cumulative [``*VCUM``].

        Array elements should not be skipped with the ``*VMASK`` and the
        NINC value of the ``*VLEN`` specifications.  The number of
        rows [``*VLEN``] applies to the Par1 array.  See the
        ``*VOPER`` command for details.

        This command is valid in any processor.
        """
        command = f"*MFUN,{parr},{func},{par1}"
        return self.run(command, **kwargs)

    def moper(
        self,
        parr="",
        par1="",
        oper="",
        val1="",
        val2="",
        val3="",
        val4="",
        val5="",
        val6="",
        **kwargs,
    ):
        """Performs matrix operations on array parameter matrices.

        APDL Command: ``*MOPER``

        Parameters
        ----------
        parr
            The name of the resulting array parameter matrix.  See ``*SET`` for
            name restrictions.

        par1
            First array parameter matrix input to the operation. For ``Oper =
            MAP``, this is an ``N`` x 3 array of coordinate locations at which to
            interpolate. ``ParR`` will then be an ``N(out)`` x ``M`` array containing the
            interpolated values.

        oper
            Matrix operations:

            * `INVERT`
              ``(*MOPER, ParR, Par1, INVERT)``
              Square matrix invert: Inverts the ``n`` x ``n`` matrix in ``Par1``
              into ``ParR``. The matrix must be well conditioned.

              .. warning::

                 Non-independent or ill-conditioned equations can
                 cause erroneous results.

              For large matrices, use the
              APDL Math operation ``*LSFACTOR`` for efficiency (see APDL
              Math).

            * `MULT`
              ``(*MOPER, ParR, Par1, MULT, Par2)``
              Matrix multiply: Multiplies ``Par1`` by ``Par2``.  The number of
              rows of ``Par2`` must equal the number of columns of ``Par1`` for
              the operation. If ``Par2`` is input with a number of rows
              greater than the number of columns of ``Par1``, matrices are
              still multiplied. However, the operation only uses a
              number of rows of ``Par2`` equal to the number of columns of
              ``Par1``.

            * `COVAR`
              ``(*MOPER, ParR, Par1, COVAR, Par2)``
              Covariance: The measure of association between two columns
              of the input matrix (``Par1``).  ``Par1``, of size m runs (rows)
              by ``n`` data (columns) is first processed to produce a row
              vector containing the mean of each column which is
              transposed to a column vector (``Par2``) of n array elements.
              The ``Par1`` and ``Par2`` operation then produces a resulting ``n`` x
              ``n`` matrix (``ParR``) of covariances (with the variances as the
              diagonal terms).

            * `CORR`
              ``(*MOPER, ParR, Par1, CORR, Par2)``
              Correlation: The correlation coefficient between two
              variables.  The input matrix (``Par1``), of size m runs (rows)
              by n data (columns), is first processed to produce a row
              vector containing the mean of each column which is then
              transposed to a column vector (``Par2``) of n array elements.
              The ``Par1`` and ``Par2`` operation then produces a resulting ``n`` x
              ``n`` matrix (``ParR``) of correlation coefficients (with a value
              of 1.0 for the diagonal terms).

            * `SOLV`
              ``(*MOPER, ParR, Par1, SOLV, Par2)``
              Solution of simultaneous equations: Solves the set of ``n``
              equations of n terms of the form ``an_1 x_1 + an_2 x_2 + ... +
              an_n x_n = b_n`` where ``Par1`` contains the matrix of
              a-coefficients, ``Par2`` the vector(s) of b-values, and ``ParR``
              the vector(s) of x-results.  ``Par1`` must be a square matrix.
              The equations must be linear, independent, and well
              conditioned.

              **Warning**: Non-independent or ill-conditioned equations can
              cause erroneous results. - For large matrices, use the
              APDL Math operation ``*LSFACTOR`` for efficiency (see APDL
              Math).

            * `SORT`
              ``(*MOPER, ParR, Par1, SORT, Par2, n1, n2, n3)``
              Matrix sort: Sorts matrix ``Par1`` according to sort vector
              ``Par2`` and places the result back in ``Par1``. Rows of ``Par1`` are
              moved to the corresponding positions indicated by the
              values of ``Par2``. ``Par2`` may be a column of ``Par1`` (in which
              case it will also be reordered). Alternatively, you may
              specify the column of ``Par1`` to sort using n1 (leaving ``Par2``
              blank). A secondary sort can be specified by column ``n2``,
              and a third sort using ``n3``. ``ParR`` is the vector of initial
              row positions (the permutation vector).  Sorting ``Par1``
              according to ``ParR`` should reproduce the initial ordering.

            * `NNEAR`
              ``(*MOPER, ParR, Par1, NNEAR, Toler)``
              Nearest Node: Quickly determine all the nodes within a
              specified tolerance of a given array.  ``ParR`` is a vector of
              the nearest selected nodes, or 0 if no nodes are nearer
              than ``Toler``. ``Par1`` is the ``n`` x 3 array of coordinate
              locations. ``Toler`` defaults to 1 and is limited to the
              maximum model size.

            * `ENEAR`
              ``(*MOPER, ``ParR``, ``Par1``, ENEAR, Toler)``
              Nearest Element: Quickly determine the elements with
              centroids that are within a specified tolerance of the
              points in a given array. - ``ParR`` is a vector of the nearest
              selected elements, or 0 if no element centroids are nearer
              than ``Toler``. ``Par1`` is the ``n`` x 3 array of coordinate
              locations.

            * `MAP`
              ``(*MOPER, ParR, Par1, MAP, Par2, Par3, kDim, --, kOut, LIMIT)``
              Maps the results from one set of points to another. For
              example, you can map pressures from a CFD analysis onto
              your model for a structural analysis.

              * ``Par1`` is the ``Nout`` x 3 array of points that will be mapped
                to. ``Par2`` is the ``Nin`` x ``M`` array that contains ``M`` values of
                data to be interpolated at each point and corresponds to
                the ``Nin`` x 3 points in ``Par3``. The resulting ``ParR`` is the ``Nout``
                x ``M`` array of mapped data points.

                For each point in the destination mesh, all possible
                triangles in the source mesh are searched to find the best
                triangle containing each point. It then does a linear
                interpolation inside this triangle. You should carefully
                specify your interpolation method and search criteria in
                order to provide faster and more accurate results (see
                ``LIMIT``, below).

              * ``kDim`` is the interpolation criteria. If ``kDim = 2 or 0``, two
                dimensional interpolation is applied (interpolate on a
                surface). If ``kDim = 3``, three dimensional interpolation is
                applied (interpolate on a volume).

              * ``kOut`` specified how points outside of the domain are
                handled. If ``kOut`` = 0, use the value(s) of the nearest
                region point for points outside of the region. If ```kOut``` =
                1, set results outside of the region to zero.

            * `LIMIT` specifies the number of nearby points considered for
              interpolation. The default is 20, and the minimum is
              5. Lower values will reduce processing time; however, some
              distorted or irregular sets of points will require a
              higher ``LIMIT`` value to encounter three nodes for
              triangulation.

              Output points are incorrect if they are not within the
              domain (area or volume) defined by the specified input
              points. Also, calculations for out-of-bound points require
              much more processing time than do points that are within
              bounds. Results mapping is available from the command line
              only.

            * `INTP`
              ``(*MOPER, ParR, Par1, INTP, Par2)``
              Finds the elements that contain each point in the array of
              ``n`` x 3 points in ``Par1``. ``Par2`` will contain the set of element
              ID numbers and ``ParR`` will contain their ``n`` x 3 set of
              natural element coordinates (values between -1 and
              1). ``Par1`` must be in global Cartesian coordinates.

            * `SGET`
              ``(*MOPER, ParR, Par1, SGET, Par2, Label, Comp)``
              Gets the nodal solution item corresponding to Label and
              Comp (see the PLNSOL command) and interpolates it to the
              given element locations. ``Par1`` contains the ``n`` x 3 array of
              natural element coordinates (values between -1 and 1) of
              the ``n`` element ID numbers in ``Par2``. ``Par1`` and ``Par2`` are
              usually the output of the ``*MOPER,,,INTP`` operation. ``ParR``
              contains the ``n`` interpolated results.

            * ``Val1, Val2, ..., Val6``
              Additional input used in the operation. The meanings of
              ``Val1`` through ``Val6`` vary depending on the specified matrix
              operation. See the description of Oper for details.
        """
        command = (
            f"*MOPER,{parr},{par1},{oper},{val1},{val2},{val3},{val4},{val5},{val6}"
        )
        return self.run(command, **kwargs)

    def mwrite(
        self, parr="", fname="", ext="", label="", n1="", n2="", n3="", **kwargs
    ):
        """Writes a matrix to a file in a formatted sequence.

        APDL Command: ``*MWRITE``

        Parameters
        ----------
        parr
            The name of the array parameter. See ``*SET`` for name restrictions.

        fname
            File name and directory path (248 characters maximum, including the
            characters needed for the directory path).  An unspecified
            directory path defaults to the working directory; in this case, you
            can use all 248 characters for the file name.

        ext
            Filename extension (eight-character maximum).

        label
            Can use a value of ``IJK``, ``IKJ``, ``JIK``, ``JKI``, ``KIJ``, ``KJI``, or blank (``JIK``).

        n1, n2, n3
            Write as ``(((ParR(i,j,k), k = 1,n1), i = 1, n2), j = 1, n3)`` for
            ``Label = KIJ``. ``n1``, ``n2``, and ``n3`` default to the corresponding dimensions
            of the array parameter ParR.

        Notes
        -----
        Writes a matrix or vector to a specified file in a formatted sequence.
        You can also use the ``*VWRITE`` command to write data to a specified file.
        Both commands contain format descriptors on the line immediately
        following the command. The format descriptors can be in either Fortran
        or C format.

        Fortran format descriptors are enclosed in parentheses. They must
        immediately follow the ``*MWRITE`` command on a separate line of the same
        input file. The word FORMAT should not be included. The format must
        specify the number of fields to be written per line, the field width,
        the placement of the decimal point, etc. There should be one field
        descriptor for each data item written. The write operation uses the
        available system FORTRAN FORMAT conventions (see your system FORTRAN
        manual). Any standard FORTRAN real format (such as (4F6.0),
        (E10.3,2X,D8.2), etc.) and character format (A) may be used.  Integer
        (I) and list-directed (``*``) descriptors may not be used. Text may be
        included in the format as a quoted string. The FORTRAN descriptor must
        be enclosed in parentheses and the format must not exceed 80 characters
        (including parentheses).

        The "C" format descriptors are used if the first character of the
        format descriptor line is not a left parenthesis. "C" format
        descriptors may be up to 80 characters long, consisting of text strings
        and predefined "data descriptors" between the strings where numeric or
        alphanumeric character data are to be inserted. The normal descriptors
        are %I for integer data, %G for double precision data, %C for
        alphanumeric character data, and %/ for a line break. There must be one
        data descriptor for each specified value in the order of the specified
        values. The enhanced formats described in ``*MSG`` may also be used.

        The starting array element number must be defined. Looping continues in
        the directions indicated by the Label argument. The number of loops and
        loop skipping may also be controlled with the ``*VLEN`` and ``*VMASK``
        commands, which work in the n2 direction (by row on the output file),
        and by  the ``*VCOL`` command, which works in the n1 direction (by column
        in the output file).  The vector specifications ``*VABS`` and ``*VFACT`` apply
        to this command, while ``*VCUM`` does not apply to this command. See the
        ``*VOPER`` command for details. If you are in the GUI, the ``*MWRITE`` command
        must be contained in an externally prepared file and read into ANSYS
        (i.e., ``*USE``, ``/INPUT``, etc.).

        This command is valid in any processor.
        """
        command = f"*MWRITE,{parr},{fname},{ext},,{label},{n1},{n2},{n3}"
        return self.run(command, **kwargs)

    def starvput(
        self,
        parr="",
        entity="",
        entnum="",
        item1="",
        it1num="",
        item2="",
        it2num="",
        kloop="",
        **kwargs,
    ):
        """Restores array parameter values into the ANSYS database.

        APDL Command: ``*VPUT``

        Parameters
        ----------
        parr
            The name of the input vector array parameter.  See ``*SET`` for name
            restrictions.  The parameter must exist as a dimensioned array
            [``*DIM``] with data input.

        entity
            Entity keyword.  Valid keywords are shown for Entity = in the table
            below.

        entnum
            The number of the entity (as shown for ENTNUM= in the table below).

        item1
            The name of a particular item for the given entity.  Valid items
            are as shown in the Item1 columns of the table below.

        it1num
            The number (or label) for the specified Item1 (if any).  Valid
            IT1NUM values are as shown in the IT1NUM columns of the table
            below.  Some Item1 labels do not require an IT1NUM value.

        item2, it2num
            A second set of item labels and numbers to further qualify the item
            for which data is to be stored.  Most items do not require this
            level of information.

        kloop
            Field to be looped on:

            Loop on the ENTNUM field (default). - Loop on the Item1 field.

            Loop on the IT1NUM field.  Successive items are as shown with IT1NUM. - Loop on the Item2 field.

        Notes
        -----
        The ``*VPUT`` command is not supported for PowerGraphics
        displays.  Inconsistent results may be obtained if this
        command is not used in /GRAPHICS, FULL.

        Plot and print operations entered via the GUI (Utility Menu>
        Pltcrtls, Utility Menu> Plot) incorporate the AVPRIN
        command. This means that the principal and equivalent values
        are recalculated. If you use ``*VPUT`` to put data back into
        the database, issue the plot commands from the command line to
        preserve your data.

        This operation is basically the inverse of the ``*VGET``
        operation.  Vector items are put directly (without any
        coordinate system transformation) into the ANSYS database.
        Items can only replace existing items of the database and not
        create new items.  Degree of freedom results that are replaced
        in the database are available for all subsequent
        postprocessing operations.  Other results are changed
        temporarily and are available mainly for the immediately
        following print and display operations.  The vector
        specification ``*VCUM`` does not apply to this command.  The
        valid labels for the location fields (Entity, ENTNUM, Item1,
        and IT1NUM) are listed below.  Item2 and IT2NUM are not
        currently used.  Not all items from the ``*VGET`` list are
        allowed on ``*VPUT`` since putting values into some locations
        could cause the database to be inconsistent.

        This command is valid in any processor.

        """
        command = (
            f"*VPUT,{parr},{entity},{entnum},{item1},{it1num},{item2},{it2num},{kloop}"
        )
        return self.run(command, **kwargs)

    def sread(
        self,
        strarray="",
        fname="",
        ext="",
        nchar="",
        nskip="",
        nread="",
        **kwargs,
    ):
        """Reads a file into a string array parameter.

        APDL Command: ``*SREAD``

        Parameters
        ----------
        strarray
            Name of the "string array" parameter which will hold the read file.
            String array parameters are similar to character arrays, but each
            array element can be as long as 128 characters. If the string
            parameter does not exist, it will be created. The array will be
            created as: ``*DIM,StrArray,STRING,nChar,nRead```

        fname
            File name and directory path (248 characters maximum, including the
            characters needed for the directory path).  An unspecified
            directory path defaults to the working directory; in this case, you
            can use all 248 characters for the file name.

        ext
            Filename extension (eight-character maximum).

        nchar
            Number of characters per line to read (default is length of the
            longest line in the file).

        nskip
            Number of lines to skip at the start of the file (default is 0).

        nread
            Number of lines to read from the file (default is the entire file).

        Notes
        -----
        The ``*SREAD`` command reads from a file into a string array
        parameter. The file must be an ASCII text file.
        """
        command = f"*SREAD,{strarray},{fname},{ext},,{nchar},{nskip},{nread}"
        return self.run(command, **kwargs)

    def toper(
        self,
        parr="",
        par1="",
        oper="",
        par2="",
        fact1="",
        fact2="",
        con1="",
        **kwargs,
    ):
        """Operates on table parameters.

        APDL Command: ``*TOPER``

        Parameters
        ----------
        parr
            Name of the resulting table parameter. The command will create a
            table array parameter with this name.  Any existing parameter with
            this name will be overwritten.

        par1
            Name of the first table parameter.

        oper
            The operation to be performed: ``ADD``.  The operation is:
            ``ParR(i,j,k) = FACT1*Par1(i,j,k) + FACT2 *Par2(i,j,k) +CON1``

        par2
            Name of the second table parameter.

        fact1
            The first table parameter multiplying constant. Defaults to 1.

        fact2
            The second table parameter multiplying constant.  Defaults to 1.

        con1
            The constant increment for offset.  Defaults to 0.

        Notes
        -----
        ``*TOPER`` operates on table parameters according to:
        ``ParR(i,j,k) = FACT1*Par1(i,j,k) + FACT2 *Par2(i,j,k) +CON1``

        Par1 and Par2 must have the same dimensions and the same variable names
        corresponding to those dimensions. Par1 and Par2 must also have
        identical index values for rows, columns, etc.

        If you want a local coordinate system for the resulting array, you must
        dimension it as such using the ``*DIM`` command before issuing ``*TOPER``.

        This command is valid in any processor.
        """
        command = f"*TOPER,{parr},{par1},{oper},{par2},{fact1},{fact2},{con1}"
        return self.run(command, **kwargs)

    def vabs(self, kabsr="", kabs1="", kabs2="", kabs3="", **kwargs):
        """Applies the absolute value function to array parameters.

        APDL Command: ``*VABS``

        Parameters
        ----------
        kabsr
            Absolute value of results parameter:

            Do not take absolute value of results parameter (``ParR``). - Take absolute value.

        kabs1
            Absolute value of first parameter:

            Do not take absolute value of first parameter (``Par1`` or ``ParI``). - Take absolute value.

        kabs2
            Absolute value of second parameter:

            Do not take absolute value of second parameter (``Par2`` or ``ParJ``). - Take absolute value.

        kabs3
            Absolute value of third parameter:

            Do not take absolute value of third parameter (``Par3`` or ``ParK``). - Take absolute value.

        Notes
        -----
        Applies an absolute value to parameters used in certain  ``*VXX`` and ``*MXX``
        operations.  Typical absolute value applications are of the form:

        ``ParR = |f(|Par1|)|``

        or

        ``ParR = |(|Par1| o |Par2|)|``

        The absolute values are applied to each input parameter value before
        the operation and to the result value after the operation.  Absolute
        values are applied before the scale factors so that negative scale
        factors may be used.  The absolute value settings are reset to the
        default (no absolute value) after each ``*VXX`` or ``*MXX`` operation.  Use
        ``*VSTAT`` to list settings.

        This command is valid in any processor.
        """
        command = f"*VABS,{kabsr},{kabs1},{kabs2},{kabs3}"
        return self.run(command, **kwargs)

    def vcol(self, ncol1="", ncol2="", **kwargs):
        """Specifies the number of columns in matrix operations.

        APDL Command: ``*VCOL``

        Parameters
        ----------
        ncol1
            Number of columns to be used for Par1 with ``*MXX`` operations.
            Defaults to whatever is needed to fill the result array.

        ncol2
            Number of columns to be used for Par2 with ``*MXX`` operations.
            Defaults to whatever is needed to fill the result array.

        Notes
        -----
        Specifies the number of columns to be used in array parameter matrix
        operations.  The size of the submatrix used is determined from the
        upper left starting array element (defined on the operation command) to
        the lower right array element (defined by the number of columns on this
        command and the number of rows on the ``*VLEN`` command).

        The default NCOL is calculated from the maximum number of columns of
        the result array (the ``*DIM`` column dimension) minus the starting
        location + 1.  For example, ``*DIM,R,,1,10`` and a starting location of
        R(1,7) gives a default of 4 columns ( starting with R(1,7), R(1,8),
        R(1,9), and R(1,10)).  Repeat operations automatically terminate at the
        last column of the result array.  Existing values in the rows and
        columns of the results matrix remain unchanged where not overwritten by
        the requested input or operation values.

        The column control settings are reset to the defaults after each ``*MXX``
        operation.  Use ``*VSTAT`` to list settings.

        This command is valid in any processor.
        """
        command = f"*VCOL,{ncol1},{ncol2}"
        return self.run(command, **kwargs)

    def vcum(self, key="", **kwargs):
        """Allows array parameter results to add to existing results.

        APDL Command: ``*VCUM``

        Parameters
        ----------
        key
            Accumulation key:

            Overwrite results. - Add results to the current value of the results parameter.

        Notes
        -----
        Allows results from certain ``*VXX`` and ``*MXX`` operations to overwrite or
        add to existing results.  The cumulative operation is of the form:

        ``ParR = ParR + ParR(Previous)``

        The cumulative setting is reset to the default (overwrite) after each
        ``*VXX`` or ``*MXX`` operation.  Use ``*VSTAT`` to list settings.

        This command is valid in any processor.
        """
        command = f"*VCUM,{key}"
        return self.run(command, **kwargs)

    def vfact(self, factr="", fact1="", fact2="", fact3="", **kwargs):
        """Applies a scale factor to array parameters.

        APDL Command: ``*VFACT``

        Parameters
        ----------
        factr
            Scale factor applied to results (``ParR``) parameter.  Defaults to 1.0.

        fact1
            Scale factor applied to first parameter (``Par1`` or ``ParI``).  Defaults
            to 1.0.

        fact2
            Scale factor applied to second parameter (``Par2`` or ``ParJ``).  Defaults
            to 1.0.

        fact3
            Scale factor applied to third parameter (``Par3`` or ``ParK``).  Defaults
            to 1.0.

        Notes
        -----
        Applies a scale factor to parameters used in certain ``*VXX`` and ``*MXX``
        operations.   Typical scale factor applications are of the form:

        ``ParR = FACTR*f(FACT1*Par1)``

        or

        ``ParR = FACTR*((FACT1*Par1) o (FACT2*Par2))``

        The factors are applied to each input parameter value before the
        operation and to the result value after the operation.  The scale
        factor settings are reset to the default (1.0) after each ``*VXX`` or ``*MXX``
        operation.  Use ``*VSTAT`` to list settings.

        This command is valid in any processor.
        """
        command = f"*VFACT,{factr},{fact1},{fact2},{fact3}"
        return self.run(command, **kwargs)

    def vfun(self, parr="", func="", par1="", con1="", con2="", con3="", **kwargs):
        """Performs a function on a single array parameter.

        APDL Command: ``*VFUN``

        Parameters
        ----------
        parr
            The name of the resulting numeric array parameter vector.  See ``*SET``
            for name restrictions.

        func
            Function to be performed:

            * ACOS
              Arccosine: ``ACOS(Par1)``.

            * ``ASIN``
              Arcsine: ``ASIN(Par1)``.

            * ``ASORT``
              ``Par1`` is sorted in ascending order.  ``*VCOL``, ``*VMASK``,
              ``*VCUM``, and ``*VLEN,,NINC`` do not apply.
              ``*VLEN,NROW`` does apply.

            * ``ATAN``
              Arctangent: ``ATAN(Par1)``

            * ``COMP``
              Compress: Selectively compresses data set.  "True"
              (``*VMASK``) values of ``Par1`` (or row positions to be considered
              according to the ``NINC`` value on the ``*VLEN`` command) are
              written in compressed form to ``ParR``, starting at the
              specified position.

            * ``COPY``
              Copy: ``Par1`` copied to ``ParR``.

            * ``COS``
              Cosine: ``COS(Par1)``.

            * ``COSH``
              Hyperbolic cosine: ``COSH(Par1)``.

            * ``DIRCOS``
              Direction cosines of the principal stresses (nX9).  ``Par1``
              contains the nX6 component stresses for the ``n`` locations of
              the calculations.

            * ``DSORT``
              ``Par1`` is sorted in descending order.
              ``*VCOL``, ``*VMASK``, ``*VCUM``, and ``*VLEN,,NINC`` do not apply.
              ``*VLEN,NROW`` does apply.

            * ``EURLER``
              Euler angles of the principal stresses (nX3).  ``Par1``
              contains the nX6 component stresses for the ``n`` locations of
              the calculations.

            * ``EXP``
              Exponential: ``EXP(Par1)``.

            * ``EXPA``
              Expand: Reverse of the COMP function.  All elements of
              ``Par1`` (starting at the position specified) are written in
              expanded form to corresponding "true" (``*VMASK``) positions
              (or row positions to be considered according to the ``NINC``
              value on the ``*VLEN`` command) of ``ParR``.

            * ``LOG``
              Natural logarithm: ``LOG(Par1)``.

            * ``LOG10``
              Common logarithm: ``LOG10(Par1)``.

            * ``NINT``
              Nearest integer: 2.783 becomes 3.0, -1.75 becomes -2.0.

            * ``NOT``
              Logical complement: values 0.0 (false) become 1.0 (true).
              Values > 0.0 (true) become 0.0 (false).

            * ``PRIN``
              Principal stresses (nX5). ``Par1`` contains the ``nX6`` component stresses
              for the ``n`` locations of the calculations.

            * ``PWR``
              Power function: ``Par1**CON1``. Exponentiation of any negative
              number in the vector ``Par1`` to a non-integer power is
              performed by exponentiating the positive number and
              prepending the minus sign. For example, ``-4**2.3`` is
              ``-(4**2.3)``.

            * ``SIN``
              Sine: ``SIN(Par1)``

            * ``SINH``
              Hyperbolic sine: ``SINH(Par1)``.

            * ``SQRT``
              Square root: ``SQRT(Par1)``.

            * ``TAN``
              Tangent: ``TAN(Par1)``.

            * ``TANH``
              Hyperbolic tangent: ``TANH(Par1)``.

            * ``TANG``
              Tangent to a path at a point: the slope at a point is determined by linear interpolation half
              way between the previous and next points. Points are assumed to be in the global Cartesian
              coordinate system. Path points are specified in array ``Par1`` (having 3 consecutive columns
              of data, with the columns containing the ``x``, ``y``, and ``z`` coordinate locations, respectively, of
              the points). Only the starting row index and the column index for the x coordinates are
              specified, such as ``A(1,1)``. The y and z coordinates of the vector are assumed to begin in the
              corresponding next columns, such as ``A(1,2)`` and ``A(1,3)``. The tangent result, ``ParR``, must also
              have 3 consecutive columns of data and will contain the tangent direction vector (normalized
              to 1.0); such as 1,0,0 for an x-direction vector.

            * ``NORM``
              Normal to a path and an input vector at a point: determined from the cross-product of the
              calculated tangent vector (see ``TANG``) and the input direction vector (with the ``i``,
              ``j``, and ``k`` components input as ``CON1``, ``CON2``, and ``CON3``).
              Points are assumed to be in the global Cartesian coordinate system.
              Path points are specified in array ``Par1`` (having 3 consecutive
              columns of data, with the columns containing the ``x``, ``y``, and ``z``
              coordinate locations, respectively, of the points).
              Only the starting row index and the column index for the ``x`` coordinates
              are specified, such as ``A(1,1)``. The ``y`` and ``z`` coordinates of the vector are assumed to begin
              in the corresponding next columns, such as ``A(1,2)`` and ``A(1,3)``. The normal result, ``ParR``,
              must also have 3 consecutive columns of data and will contain the normal direction vector
              (normalized to 1.0); such as ``1,0,0`` for an x-direction vector

            * ``LOCAL``
              Transforms global Cartesian coordinates of a point to the coordinates of a specified system:
              points to be transformed are specified in array ``Par1`` (having 3 consecutive columns of
              data, with the columns containing the ``x``, ``y``, and ``z`` global Cartesian coordinate locations,
              respectively, of the points).
              Only the starting row index and the column index for the ``x``
              coordinates are specified, such as ``A(1,1)``.
              The y and z coordinates of the vector are assumed
              to begin in the corresponding next columns, such as ``A(1,2)`` and ``A(1,3)``.
              Results are transformed to coordinate system ``CON1`` (which may be any valid coordinate system number,
              such as 1,2,11,12, etc.). The transformed result, ``ParR``, must also have 3 consecutive columns
              of data and will contain the corresponding transformed coordinate locations.

            * ``GLOBAL``
              Transforms specified coordinates of a point to global Cartesian coordinates: points to be
              transformed are specified in array ``Par1`` (having 3 consecutive columns of data, with the
              columns containing the local coordinate locations (``x``, ``y``, ``z`` or ``r``, ``θ``, ``z``
              or etc.) of the points).
              Only the starting row index and the column index for the x coordinates are specified, such
              as ``A(1,1)``.
              The y and z coordinates (or ``θ`` and ``z``, or etc.) of the vector are assumed to begin
              in the corresponding next columns, such as ``A(1,2)`` and ``A(1,3)``.
              Local coordinate locations are assumed to be in coordinate system ``CON1``
              (which may be any valid coordinate system number, such as 1,2,11,12, etc.).
              The transformed result, ``ParR``, must also have 3 consecutive
              columns of data, with the columns containing the global Cartesian ``x``, y, and z coordinate
              locations, respectively.

        par1
            Array parameter vector in the operation.

        con1, con2, con3
            Constants (used only with the PWR, NORM, LOCAL, and GLOBAL
            functions).

        Notes
        -----
        Operates on one input array parameter vector and produces one output
        array parameter vector according to:

        ``ParR = f(Par1)``

        """
        command = f"*VFUN,{parr},{func},{par1},{con1},{con2},{con3}"
        return self.run(command, **kwargs)

    def vitrp(self, parr="", part="", pari="", parj="", park="", **kwargs):
        """Forms an array parameter by interpolation of a table.

        APDL Command: ``*VITRP``

        Parameters
        ----------
        parr
            The name of the resulting array parameter.  See ``*SET`` for name
            restrictions.

        part
            The name of the TABLE array parameter.  The parameter must exist as
            a dimensioned array of type TABLE [``*DIM``].

        pari
            Array parameter vector of I (row) index values for interpolation in
            ParT.

        parj
            Array parameter vector of J (column) index values for interpolation
            in ParT (which must be at least 2-D).

        park
            Array parameter vector of K (depth) index values for interpolation
            in ParT (which must be 3-D).

        Notes
        -----
        Forms an array parameter (of type ``ARRAY``) by interpolating values of an
        array parameter (of type ``TABLE``) at specified table index locations
        according to:

        ``ParR = f(ParT, Parl, ParJ, ParK)``

        where ``ParT`` is the type ``TABLE`` array parameter, and ``ParI``, ``ParJ``, ``ParK`` are
        the type ``ARRAY`` array parameter vectors of index values for
        interpolation in ``ParT``.  See the ``*DIM`` command for ``TABLE`` and ``ARRAY``
        declaration types.  Linear interpolation is used.  The starting array
        element number for the ``TABLE`` array (``ParT``) is not used (but a value must
        be input).  Starting array element numbers must be defined for each
        array parameter vector if it does not start at the first location. For
        example, ``*VITRP,R(5),TAB(1,1),X(2),Y(4)`` uses the second element of X
        and the fourth element of Y as index values (row and column) for a 2-D
        interpolation in ``TAB`` and stores the result in the fifth element of R.
        Operations continue on successive array elements ``[*VLEN, *VMASK]`` with
        the default being all successive elements.  Absolute values and scale
        factors may be applied to the result parameter ``[*VABS, *VFACT]``.
        Results may be cumulative ``[*VCUM]``.  See the ``*VOPER`` command for details.

        This command is valid in any processor.
        """
        command = f"*VITRP,{parr},{part},{pari},{parj},{park}"
        return self.run(command, **kwargs)

    def vlen(self, nrow="", ninc="", **kwargs):
        """Specifies the number of rows to be used in array parameter operations.

        APDL Command: ``*VLEN``

        Parameters
        ----------
        nrow
            Number of rows to be used with the ``*VXX`` or ``*MXX`` operations.
            Defaults to the number of rows needed to fill the result array.

        ninc
            Perform the operation on every NINC row (defaults to 1).

        Notes
        -----
        Specifies the number of rows to be used in array parameter operations.
        The size of the submatrix used is determined from the upper left
        starting array element (defined on the operation command) to the lower
        right array element (defined by the number of rows on this command and
        the number of columns on the ``*VCOL`` command).  ``NINC`` allows skipping row
        operations for some operation commands.  Skipped rows are included in
        the row count.  The starting row number must be defined on the
        operation command for each parameter read and for the result written.

        The default ``NROW`` is calculated from the maximum number of rows of the
        result array (the ``*DIM`` row dimension) minus the starting location + 1.
        For example, ``*DIM,R,,10`` and a starting location of ``R(7)`` gives a default
        of 4 loops (filling ``R(7)``, ``R(8)``, ``R(9)``, and ``R(10)``).  Repeat operations
        automatically terminate at the last row of the result array.  Existing
        values in the rows and columns of the results matrix remain unchanged
        where not overwritten by the requested input or operation values.

        The stride (``NINC``) allows operations to be performed at regular
        intervals.  It has no effect on the total number of row operations.
        Skipped operations retain the previous result.  For example, ``*DIM,R,,6,``
        with a starting location of ``R(1)``, ``NROW = 10``, and ``NINC = 2`` calculates
        values for locations ``R(1)``, ``R(3)``, and ``R(5)`` and retains values for
        locations ``R(2)``, ``R(4)``, and ``R(6)``.  A more general skip control may be
        done by masking ``[*VMASK]``.  The row control settings are reset to the
        defaults after each ``*VXX`` or ``*MXX`` operation.  Use ``*VSTAT`` to list
        settings.

        This command is valid in any processor.
        """
        command = f"*VLEN,{nrow},{ninc}"
        return self.run(command, **kwargs)

    def vmask(self, par="", **kwargs):
        """Specifies an array parameter as a masking vector.

        APDL Command: ``*VMASK``

        Parameters
        ----------
        par
            Name of the mask parameter.  The starting subscript must also be
            specified.

        Notes
        -----
        Specifies the name of the parameter whose values are to be checked for
        each resulting row operation.  The mask vector usually contains only 0
        (for false) and 1 (for true) values.  For each row operation the
        corresponding mask vector value is checked.  A true value allows the
        operation to be done.  A false value skips the operation (and retains
        the previous results).  A mask vector can be created from direct input,
        such as ``M(1) = 1,0,0,1,1,0,1``; or from the DATA function of the ``*VFILL``
        command.  The NOT function of the ``*VFUN`` command can be used to reverse
        the logical sense of the mask vector.  The logical compare operations
        (``LT``, ``LE``, ``EQ``, ``NE``, ``GE``, and ``GT``) of
        the ``*VOPER`` command also produce a mask
        vector by operating on two other vectors.  Any numeric vector can be
        used as a mask vector since the actual interpretation assumes values
        less than 0.0 are 0.0 (false) and values greater than 0.0 are 1.0
        (true).  If the mask vector is not specified (or has fewer values than
        the result vector), true (1.0) values are assumed for the unspecified
        values.  Another skip control may be input with ``NINC`` on the ``*VLEN``
        command.  If both are present, operations occur only when both are
        true.  The mask setting is reset to the default (no mask) after each
        ``*VXX`` or ``*MXX`` operation.  Use ``*VSTAT`` to list settings.

        This command is valid in any processor.
        """
        command = f"*VMASK,{par}"
        return self.run(command, **kwargs)

    def voper(self, parr="", par1="", oper="", par2="", con1="", con2="", **kwargs):
        """Operates on two array parameters.

        APDL Command: ``*VOPER``

        Parameters
        ----------
        PARR
            The name of the resulting array parameter vector.  See ``*SET``  for
            name restrictions.

        PAR1
            First array parameter vector in the operation.  May also be a
            scalar parameter or a literal constant.

        OPER
            Operations:

            * ``ADD``
              Addition: ``Par1+Par2``.

            * ``SUB``
              Subtraction: ``Par1-Par2``.

            * ``MULT``
              Multiplication: ``Par1*Par2``.

            * ``DIV``
              Division: ``Par1/Par2``  (a divide by zero results in a value of zero).

            * ``MIN``
              Minimum: minimum of ``Par1`` and ``Par2``.

            * ``MAX``
              Maximum: maximum of ``Par1`` and ``Par2``.

            * ``LT``
              Less than comparison: ``Par1<Par2`` gives 1.0 if true, 0.0 if
              false.

            * ``LE``
              Less than or equal comparison: ``Par1 <= Par2`` gives
              1.0 if true, 0.0 if false.

            * ``EQ``
              Equal comparison: ``Par1 = Par2`` gives 1.0 if true, 0.0 if
              false.

            * ``NE``
              Not equal comparison: ``Par1 ≠ Par2`` gives 1.0 if
              true, 0.0 if false.

            * ``GE``
              Greater than or equal comparison: ``Par1 >= Par2`` gives 1.0 if
              true, 0.0 if false.

            * ``GT``
              Greater than comparison: ``Par1>Par2``
              gives 1.0 if true, 0.0 if false.

            * ``DER``
              First derivative:

              .. math::

                 \\dfrac{\\mathrm{d}(\\mathrm{Par1})}{\\mathrm{d}(\\mathrm{Par2})}

              The derivative at a point is determined over points
              half way between the previous and next points
              (by linear interpolation).
              ``Par1`` must be a function (a unique ``Par1``
              value for each ``Par2``
              value) and ``Par2`` must be in ascending order.

            * ``DER2``
              Second derivative:

              .. math::

                 \\dfrac{\\mathrm{d}^2(\\mathrm{Par1})}{\\mathrm{d}(\\mathrm{Par2})^2}

              See also ``DER1``.

            * ``INT1``
              Single integral:

              .. math::

                 \\int Par1 \\, d(Par2)

              where ``CON1`` is the integration constant.
              The integral at a point is
              determined by using the single integration procedure
              described in the Mechanical APDL Theory Reference.

            * ``INT2``
              Double integral:

              .. math::

                 \\iint Par1 \\, d(Par2)

              where ``CON1`` is the integration constant of the first
              integral and ``CON2`` is the integration constant
              of the second integral.  If ``Par1``
              contains acceleration data, ``CON1`` is the initial velocity
              and ``CON2`` is the initial displacement.  See also ``INT1``.

            * ``DOT``
              Dot product: ``Par1 . Par2``.
              ``Par1`` and ``Par2`` must each have
              three consecutive columns of data, with the columns
              containing the ``i``, ``j``, and ``k`` vector components,
              respectively.  Only the starting row index and the column
              index for the ``i`` components are specified for ``Par1`` and
              ``Par2``, such as ``A(1,1)``.  The ``j`` and ``k`` components of the
              vector are assumed to begin in the corresponding next
              columns, such as ``A(1,2)`` and ``A(1,3)``.

            * ``CROSS``
              Cross product: ``Par1 x Par2``.
              ``Par1``, ``Par2``, and ``ParR`` must each have 3 components,
              respectively.  Only the starting row index and the column
              index for the i components are specified for ``Par1``, ``Par2``,
              and ``ParR``, such as ``A(1,1)``.  The j and k components of the
              vector are assumed to begin in the corresponding next
              columns, such as ``A(1,2)`` and ``A(1,3)``.

            * ``GATH``
              Gather: For a vector of position numbers, ``Par2``, copy the
              value of ``Par1`` at each position number to ParR.  Example:
              for ``Par1 = 10,20,30,40`` and ``Par2 = 2,4,1``; ``ParR =
              20,40,10``.

            * ``SCAT``
              Scatter: Opposite of ``GATH`` operation.  For a
              vector of position numbers, ``Par2``, copy the value of ``Par1``
              to that position number in ``ParR``.  Example: for ``Par1 =
              10,20,30,40,50`` and ``Par2 = 2,1,0,5,3``; ``ParR = 20,10,50,0,40``.

            * ``ATN2``
              Arctangent: arctangent of ``Par1/Par2`` with the sign of each
              component considered.

            * ``LOCAL``
              Transform the data in ``Par1`` from
              the global Cartesian coordinate system to the local
              coordinate system given in ``CON1``. ``Par1`` must be an ``N`` x 3
              (i.e., vector) or an ``N`` x 6 (i.e., stress or strain tensor)
              array. If the local coordinate system is a cylindrical,
              spherical, or toroidal system, then you must provide the
              global Cartesian coordinates in ``Par2`` as an ``N`` x 3 array.
              Set ``CON2 = 1`` if the data is strain data.

            * ``GLOBAL``
              Transform the data in ``Par1`` from the local coordinate system given in ``CON1`` to the global
              Cartesian coordinate system. ``Par1`` must be an ``N`` x 3 (that is, vector) or an ``N`` x 6 (that is,
              stress or strain tensor) array. If the local coordinate system is a cylindrical, spherical, or
              toroidal system, then you must provide the global Cartesian coordinates in ``Par2`` as an ``N``
              x 3 array. Set ``CON2 = 1`` if the data is strain data.

        PAR2
            Second array parameter vector in the operation.  May also be a
            scalar parameter or a literal constant.

        CON1
            First constant (used only with the ``INT1`` and ``INT2`` operations).

        CON2
            Second constant (used only with the ``INT2`` operation).

        Notes
        -----
        Operates on two input array parameter vectors and produces one output
        array parameter vector according to:

        ``ParR = Par1 o Par2``

        where the operations (o) are described below.  ParR may be the same as
        ``Par1`` or ``Par2``.  Absolute values and scale factors may be applied to all
        parameters [``*VABS``, ``*VFACT``].  Results may be cumulative [``*VCUM``].
        Starting array element numbers must be defined for each array parameter
        vector if it does not start at the first location, such as
        ``*VOPER,A,B(5),ADD,C(3)`` which adds the third element of C to the fifth
        element of B and stores the result in the first element of A.
        Operations continue on successive array elements ``[*VLEN, *VMASK]`` with
        the default being all successive elements.  Skipping array elements via
        ``*VMASK`` or ``*VLEN`` for the ``DER`` and ``INT`` functions skips only the writing
        of the results (skipped array element data are used in all
        calculations).

        Parameter functions and operations are available to operate on a scalar
        parameter or a single element of an array parameter, such as ``SQRT(B)`` or
        ``SQRT(A(4))``.  See the ``*SET`` command for details.  Operations on a
        sequence of array elements can be done by repeating the desired
        function or operation in a do-loop ``[*DO]``.  The vector operations within
        the ANSYS program (``*VXX`` commands) are internally programmed do-loops
        that conveniently perform the indicated operation over a sequence of
        array elements.  If the array is multidimensional, only the first
        subscript is incremented in the do-loop, that is, the operation repeats
        in column vector fashion "down" the array.  For example, for ``A(1,5)``,
        ``A(2,5)``, ``A(3,5)``, etc.  The starting location of the row index must be
        defined for each parameter read and for the result written.

        The default number of loops is from the starting result location to the
        last result location and can be altered with the ``*VLEN`` command.  A
        logical mask vector may be defined to control at which locations the
        operations are to be skipped [``*VMASK``].  The default is to skip no
        locations.  Repeat operations automatically terminate at the last array
        element of the result array column if the number of loops is undefined
        or if it exceeds the last result array element.  Zeroes are used in
        operations for values read beyond the last array element of an input
        array column.  Existing values in the rows and columns of the results
        matrix
        """
        command = f"*VOPER,{parr},{par1},{oper},{par2},{con1},{con2}"
        return self.run(command, **kwargs)

    def vscfun(self, parr="", func="", par1="", **kwargs):
        """Determines properties of an array parameter.

        APDL Command: ``*VSCFUN``

        Parameters
        ----------
        parr
            The name of the resulting scalar parameter.  See ``*SET`` for name
            restrictions.

        func
            Functions:

            * ``MAX``
              Maximum: the maximum ``Par1`` array element value.

            * ``MIN``
              Minimum: the minimum ``Par1`` array element value.

            * ``LMAX``
              Index location of the maximum ``Par1`` array element value.
              Array ``Par1`` is searched starting from its specified
              index.

            * ``LMIN``
              Index location of the minimum ``Par1`` array element
              value.  Array ``Par1`` is searched starting from its specified
              index.

            * ``FIRST``
              Index location of the first nonzero value in array ``Par1``.
              Array ``Par1`` is searched starting from its specified
              index.

            * ``LAST``
              Index location of the last nonzero value in array
              ``Par1``.  Array ``Par1`` is searched starting from its specified
              index.

            * ``SUM``
              Sum: ``Par1`` (the summation of the ``Par1`` array element
              values).

            * ``MEDI``
              Median: value of ``Par1`` at which there are an
              equal number of values above and below.

            * ``MEAN``
              Mean: ``(σ Par1)/NUM``, where ``NUM`` is the number of summed
              values.

            * ``VARI``
              Variance: ``(σ ((Par1-MEAN)**2))/NUM``.

            * ``STDV``
              Standard deviation: square root of ``VARI``.

            * ``RMS``
              Root-mean-square: square root of ``(σ (Par1**2))/NUM``.

            * ``NUM``
              Number: the number of summed values (masked values are not counted).

        par1
            Array parameter vector in the operation.

        Notes
        -----
        Operates on one input array parameter vector and produces one output
        scalar parameter according to:

        ``ParR = f(Par1)``

        where the functions (f) are described below. The starting array element
        number must be defined for the array parameter vector.  For example,
        ``*VSCFUN,MU,MEAN,A(1)`` finds the mean of the ``A`` vector values, starting
        from the first value and stores the result as parameter ``MU``.  Operations
        use successive array elements ``[*VLEN, *VMASK]`` with the default being
        all successive array elements.  Absolute values and scale factors may
        be applied to all parameters ``[*VABS, *VFACT]``.  Results may be
        cumulative ``[*VCUM]``.  See the ``*VOPER`` command for details.

        This command is valid in any processor.
        """
        command = f"*VSCFUN,{parr},{func},{par1}"
        return self.run(command, **kwargs)

    def vstat(self, **kwargs):
        """Lists the current specifications for the array parameters.

        APDL Command: ``*VSTAT``

        Notes
        -----
        Lists the current specifications for the ``*VABS``, ``*VCOL``,
        ``*VCUM``, ``*VFACT``, ``*VLEN``, and ``*VMASK`` commands.

        This command is valid in any processor.
        """
        command = f"*VSTAT,"
        return self.run(command, **kwargs)

    def vwrite(
        self,
        par1="",
        par2="",
        par3="",
        par4="",
        par5="",
        par6="",
        par7="",
        par8="",
        par9="",
        par10="",
        par11="",
        par12="",
        par13="",
        par14="",
        par15="",
        par16="",
        par17="",
        par18="",
        par19="",
        **kwargs,
    ):
        """Writes data to a file in a formatted sequence.

        APDL Command: ``*VWRITE``

        .. warning::
           This command cannot be run interactively.  See
           :func:`non_interactive <ansys.mapdl.core.Mapdl.non_interactive>`.

        Parameters
        ----------
        par1, par2, par3, . . . , par19
            You can write up to 19 parameters (or constants) at a time. Any Par
            values after a blank  Par value are ignored.  If you leave them all
            blank, one line will be written (to write a title or a blank line).
            If you input the keyword SEQU, a sequence of numbers (starting from
            1) will be written for that item.

        Notes
        -----
        You use ``*VWRITE`` to write data to a file in a formatted sequence. Data
        items (Par1, Par2, etc.) may be array parameters, scalar parameters,
        character parameters (scalar or array), or constants.  You must
        evaluate expressions and functions in the data item fields before using
        the ``*VWRITE`` command, since initially they will be evaluated to a
        constant and remain constant throughout the operation.  Unless a file
        is defined with the ``*CFOPEN``  command, data is written to the standard
        output file. Data written to the standard output file may be diverted
        to a different file by first switching the current output file with the
        /OUTPUT command. You can also use the ``*MWRITE`` command to write data to
        a specified file. Both commands contain format descriptors on the line
        immediately following the command. The format descriptors can be in
        either Fortran or C format.

        You must enclose Fortran format descriptors in parentheses. They must
        immediately follow the ``*VWRITE`` command on a separate line of the same
        input file.  Do not include the word FORMAT. The format must specify
        the number of fields to be written per line, the field width, the
        placement of the decimal point, etc.  You should use one field
        descriptor for each data item written.  The write operation uses your
        system's available FORTRAN FORMAT conventions (see your system FORTRAN
        manual).  You can use any standard FORTRAN real format (such as
        (4F6.0), (E10.3,2X,D8.2), etc.) and alphanumeric format (A).
        Alphanumeric strings are limited to a maximum of 8 characters for any
        field (A8) using the Fortran format. Use the "C" format for string
        arrays larger than 8 characters. Integer (I) and list-directed (*)
        descriptors may not be used.  You can include text in the format as a
        quoted string.  The parentheses must be included in the format and the
        format must not exceed 80 characters (including parentheses).  The
        output line length is limited to 128 characters.

        The "C" format descriptors are used if the first character of the
        format descriptor line is not a left parenthesis. "C" format
        descriptors are up to 80 characters long, consisting of text strings
        and predefined "data descriptors" between the strings where numeric or
        alphanumeric character data will be inserted.  The normal descriptors
        are %I for integer data, %G for double precision data, %C for
        alphanumeric character data, and %/ for a line break. There must be one
        data descriptor for each specified value (8 maximum) in the order of
        the specified values. The enhanced formats described in ``*MSG`` may also
        be used.

        For array parameter items, you must define the starting array
        element number. Looping continues (incrementing the vector
        index number of each array parameter by one) each time you
        output a line, until the maximum array vector element is
        written.  For example, ``*VWRITE,A(1)`` followed by (F6.0)
        will write one value per output line, i.e., A(1), A(2), A(3),
        A(4), etc.  You write constants and scalar parameters with the
        same values for each loop.  You can also control the number of
        loops and loop skipping with the ``*VLEN`` and ``*VMASK``
        commands.  The vector specifications ``*VABS``, ``*VFACT``,
        and ``*VCUM`` do not apply to this command.  If looping
        continues beyond the supplied data array's length, zeros will
        be output for numeric array parameters and blanks for
        character array parameters.  For multi-dimensioned array
        parameters, only the first (row) subscript is incremented.
        See the ``*VOPER`` command for details.  If you are in the GUI,
        the ``*VWRITE`` command must be contained in an externally
        prepared file and read into ANSYS (i.e., ``*USE``, /INPUT, etc.).

        This command is valid in any processor.

        """
        command = f"*VWRITE,{par1},{par2},{par3},{par4},{par5},{par6},{par7},{par8},{par9},{par10},{par11},{par12},{par13},{par14},{par15},{par16},{par17},{par18},{par19}"
        return self.run(command, **kwargs)



# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class MultiFieldConvergenceControls:
    def mfconv(self, lab="", toler="", minref="", **kwargs):
        """Sets convergence values for an ANSYS Multi-field solver analysis.

        APDL Command: MFCONV

        Parameters
        ----------
        lab
            Valid labels:

        toler
            Convergence tolerance about program calculated reference value (the
            L2 norm of the new load in a multi-field analysis). Defaults to
            0.01 (1%) for all labels. Must be less than 1.0.

        minref
            The minimum value allowed for the program calculated reference
            value. If negative, no minimum is enforced. Defaults to 1.0e-6 for
            all labels. Not available in the GUI. MINREF corresponds to
            ||ϕnew|| as defined in Set up Stagger Solution in the Coupled-Field
            Analysis Guide.

        Notes
        -----
        MFCONV sets convergence values for variables at the ANSYS
        Multi-field solver interface.

        This command is also valid in PREP7.

        See Multi-field Commands in the Coupled-Field Analysis Guide
        for a list of all ANSYS Multi-field solver commands and their
        availability for MFS and MFX analyses.

        Distributed ANSYS Restriction: This command is not supported
        in Distributed ANSYS.
        """
        return self.run(f"MFCONV,{lab},{toler},,{minref}", **kwargs)

    def mfiter(self, maxiter="", miniter="", target="", **kwargs):
        """Sets the number of stagger iterations for an ANSYS Multi-field solver

        APDL Command: MFITER
        analysis.

        Parameters
        ----------
        maxiter
            Maximum number of iterations. Defaults to 10.

        miniter
            Minimum number of iterations. Defaults to 1.

        target
            Target number of iterations. Defaults to 5.

        Notes
        -----
        The number of stagger iterations applies to each time step in an ANSYS
        Multi-field solver analysis. MINITER and TARGET are valid only when
        multi-field auto time stepping is on (MFDTIME).

        This command is also valid in PREP7.

        See Multi-field Commands in the Coupled-Field Analysis Guide for a list
        of all ANSYS Multi-field solver commands and their availability for MFS
        and MFX analyses.

        Distributed ANSYS Restriction: This command is not supported in
        Distributed ANSYS.
        """
        command = f"MFITER,{maxiter},{miniter},{target}"
        return self.run(command, **kwargs)

    def mfrelax(self, lab="", value="", option="", **kwargs):
        """Sets relaxation values for an ANSYS Multi-field solver analysis.

        APDL Command: MFRELAX

        Parameters
        ----------
        lab
            Valid labels:

        value
            Relaxation value. Defaults to 0.75 for all labels.

        option
            Valid options are:

            RELX  - Uses relaxation method for load transfer (default).

            LINT  - Uses a linear interpolation for loaf transfer.

        Notes
        -----
        MFRELAX sets relaxation values for the load transfer variables at a
        surface or volume interface. Option = RELX will usually give you a more
        stable and smooth load transfer and is suitable for strongly coupled
        problems (such as FSI problems). Option = LINT is suitable for weakly
        coupled problems because it will transfer the full load in fewer
        stagger iterations.

        See the MFFR and MFITER commands for more information on relaxation in
        the ANSYS Multi-field solver.

        This command is also valid in PREP7.

        See Multi-field Commands in the Coupled-Field Analysis Guide for a list
        of all ANSYS Multi-field solver commands and their availability for MFS
        and MFX analyses.

        Distributed ANSYS Restriction: This command is not supported in
        Distributed ANSYS.
        """
        command = f"MFRELAX,{lab},{value},{option}"
        return self.run(command, **kwargs)
